@@ -1,10 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Dark matter profiles."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import abc
 import numpy as np
 import astropy.units as u
-from ...extern import six
 from ...utils.fitting import Parameter, Parameters
 from ...spectrum.utils import integrate_spectrum
 
@@ -18,8 +16,7 @@ __all__ = [
 ]
 
 
-@six.add_metaclass(abc.ABCMeta)
-class DMProfile(object):
+class DMProfile(abc.ABC):
     """DMProfile model base class."""
 
     LOCAL_DENSITY = 0.3 * u.GeV / (u.cm ** 3)
@@ -48,10 +45,8 @@ class DMProfile(object):
         r"""Integrate squared dark matter profile numerically.
 
         .. math::
-
             F(r_{min}, r_{max}) = \int_{r_{min}}^{r_{max}}\rho(r)^2 dr
 
-
         Parameters
         ----------
         rmin, rmax : `~astropy.units.Quantity`
@@ -67,7 +62,6 @@ class NFWProfile(DMProfile):
     r"""NFW Profile.
 
     .. math::
-
         \rho(r) = \rho_s \frac{r_s}{r}\left(1 + \frac{r}{r_s}\right)^{-2}
 
     Parameters
@@ -102,7 +96,6 @@ class EinastoProfile(DMProfile):
     r"""Einasto Profile.
 
     .. math::
-
         \rho(r) = \rho_s \exp{
             \left(-\frac{2}{\alpha}\left[
             \left(\frac{r}{r_s}\right)^{\alpha} - 1\right] \right)}
@@ -149,9 +142,7 @@ class EinastoProfile(DMProfile):
 class IsothermalProfile(DMProfile):
     r"""Isothermal Profile.
 
-    .. math::
-
-        \rho(r) = \frac{\rho_s}{1 + (r/r_s)^2}
+    .. math:: \rho(r) = \frac{\rho_s}{1 + (r/r_s)^2}
 
     Parameters
     ----------
@@ -183,9 +174,7 @@ class IsothermalProfile(DMProfile):
 class BurkertProfile(DMProfile):
     r"""Burkert Profile.
 
-    .. math::
-
-        \rho(r) = \frac{\rho_s}{(1 + r/r_s)(1 + (r/r_s)^2)}
+    .. math:: \rho(r) = \frac{\rho_s}{(1 + r/r_s)(1 + (r/r_s)^2)}
 
     Parameters
     ----------
@@ -218,7 +207,6 @@ class MooreProfile(DMProfile):
     r"""Moore Profile.
 
     .. math::
-
         \rho(r) = \rho_s \left(\frac{r_s}{r}\right)^{1.16}
         \left(1 + \frac{r}{r_s} \right)^{-1.84}
 
