@@ -1,6 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Galactic radial source distribution probability density functions."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from astropy.units import Quantity
 from astropy.modeling import Fittable1DModel, Parameter
@@ -52,7 +51,7 @@ class Paczynski1990(Fittable1DModel):
     evolved = False
 
     def __init__(self, amplitude=1, r_exp=4.5, **kwargs):
-        super(Paczynski1990, self).__init__(amplitude=amplitude, r_exp=r_exp, **kwargs)
+        super().__init__(amplitude=amplitude, r_exp=r_exp, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, r_exp):
@@ -90,9 +89,7 @@ class CaseBattacharya1998(Fittable1DModel):
     evolved = True
 
     def __init__(self, amplitude=1.0, alpha=2, beta=3.53, **kwargs):
-        super(CaseBattacharya1998, self).__init__(
-            amplitude=amplitude, alpha=alpha, beta=beta, **kwargs
-        )
+        super().__init__(amplitude=amplitude, alpha=alpha, beta=beta, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, alpha, beta):
@@ -138,9 +135,7 @@ class YusifovKucuk2004(Fittable1DModel):
     evolved = True
 
     def __init__(self, amplitude=1, a=1.64, b=4.01, r_1=0.55, **kwargs):
-        super(YusifovKucuk2004, self).__init__(
-            amplitude=amplitude, a=a, b=b, r_1=r_1, **kwargs
-        )
+        super().__init__(amplitude=amplitude, a=a, b=b, r_1=r_1, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, a, b, r_1):
@@ -183,7 +178,7 @@ class YusifovKucuk2004B(Fittable1DModel):
     evolved = False
 
     def __init__(self, amplitude=1, a=4, b=6.8, **kwargs):
-        super(YusifovKucuk2004B, self).__init__(amplitude=amplitude, a=a, b=b, **kwargs)
+        super().__init__(amplitude=amplitude, a=a, b=b, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, a, b):
@@ -222,9 +217,7 @@ class FaucherKaspi2006(Fittable1DModel):
     evolved = False
 
     def __init__(self, amplitude=1, r_0=7.04, sigma=1.83, **kwargs):
-        super(FaucherKaspi2006, self).__init__(
-            amplitude=amplitude, r_0=r_0, sigma=sigma, **kwargs
-        )
+        super().__init__(amplitude=amplitude, r_0=r_0, sigma=sigma, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, r_0, sigma):
@@ -264,7 +257,7 @@ class Lorimer2006(Fittable1DModel):
     evolved = True
 
     def __init__(self, amplitude=1, B=1.9, C=5.0, **kwargs):
-        super(Lorimer2006, self).__init__(amplitude=amplitude, B=B, C=C, **kwargs)
+        super().__init__(amplitude=amplitude, B=B, C=C, **kwargs)
 
     @staticmethod
     def evaluate(r, amplitude, B, C):
@@ -302,7 +295,7 @@ class Exponential(Fittable1DModel):
     evolved = False
 
     def __init__(self, amplitude=1, z_0=0.05, **kwargs):
-        super(Exponential, self).__init__(amplitude=amplitude, z_0=z_0, **kwargs)
+        super().__init__(amplitude=amplitude, z_0=z_0, **kwargs)
 
     @staticmethod
     def evaluate(z, amplitude, z_0):
@@ -310,7 +303,7 @@ class Exponential(Fittable1DModel):
         return amplitude * np.exp(-np.abs(z) / z_0)
 
 
-class LogSpiral(object):
+class LogSpiral:
     """Logarithmic spiral.
 
     Reference: http://en.wikipedia.org/wiki/Logarithmic_spiral
@@ -402,7 +395,8 @@ class FaucherSpiral(LogSpiral):
     theta_0 = Quantity([1.57, 4.71, 4.09, 0.95], "rad")
     spiralarms = np.array(["Norma", "Carina Sagittarius", "Perseus", "Crux Scutum"])
 
-    def _blur(self, radius, theta, amount=0.07, random_state="random-seed"):
+    @staticmethod
+    def _blur(radius, theta, amount=0.07, random_state="random-seed"):
         """Blur the positions around the centroid of the spiralarm.
 
         The given positions are blurred by drawing a displacement in radius from
@@ -429,8 +423,9 @@ class FaucherSpiral(LogSpiral):
         dx, dy = cartesian(dr, dtheta)
         return polar(x + dx, y + dy)
 
+    @staticmethod
     def _gc_correction(
-        self, radius, theta, r_corr=Quantity(2.857, "kpc"), random_state="random-seed"
+        radius, theta, r_corr=Quantity(2.857, "kpc"), random_state="random-seed"
     ):
         """Correction of source distribution towards the galactic center.
 
