@@ -1,6 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """HAWC catalogs (https://www.hawc-observatory.org)."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from astropy.table import Table
 from ..utils.scripts import make_path
@@ -30,7 +29,6 @@ class SourceCatalogObject2HWC(SourceCatalogObject):
         info : {'all', 'basic', 'position', 'spectrum'}
             Comma separated list of options
         """
-
         if info == "all":
             info = "basic,position,spectrum"
 
@@ -152,7 +150,7 @@ class SourceCatalog2HWC(SourceCatalog):
     used to make the catalog, but then in the discussion considered as one source.
 
     References
-    -----------
+    ----------
     .. [1] Abeysekara et al, "The 2HWC HAWC Observatory Gamma Ray Catalog",
        On ADS: `2017ApJ...843...40A <http://adsabs.harvard.edu/abs/2017ApJ...843...40A>`__
     """
@@ -171,6 +169,4 @@ class SourceCatalog2HWC(SourceCatalog):
 
         source_name_key = "source_name"
 
-        super(SourceCatalog2HWC, self).__init__(
-            table=table, source_name_key=source_name_key
-        )
+        super().__init__(table=table, source_name_key=source_name_key)
