@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
@@ -69,7 +68,7 @@ SOURCES_3FHL = [
 ]
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestFermi3FGLObject:
     @classmethod
     def setup_class(cls):
@@ -105,10 +104,10 @@ class TestFermi3FGLObject:
 
     def test_data_python_dict(self):
         data = self.source._data_python_dict
-        assert type(data["RAJ2000"]) == float
+        assert isinstance(data["RAJ2000"], float)
         assert data["RAJ2000"] == 83.63719940185547
-        assert type(data["Unc_Flux100_300"]) == list
-        assert type(data["Unc_Flux100_300"][0]) == float
+        assert isinstance(data["Unc_Flux100_300"], list)
+        assert isinstance(data["Unc_Flux100_300"][0], float)
         assert_allclose(data["Unc_Flux100_300"][0], -1.44535601265261e-08)
 
     @requires_dependency("uncertainties")
@@ -190,7 +189,7 @@ class TestFermi3FGLObject:
         assert str(self.cat["Crab"]) == str(self.cat[name])
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestFermi1FHLObject:
     @classmethod
     def setup_class(cls):
@@ -221,7 +220,7 @@ class TestFermi1FHLObject:
         assert_quantity_allclose(actual, desired, rtol=1e-5)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestFermi2FHLObject:
     @classmethod
     def setup_class(cls):
@@ -252,7 +251,7 @@ class TestFermi2FHLObject:
         assert_quantity_allclose(actual, desired, rtol=1e-3)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestFermi3FHLObject:
     @classmethod
     def setup_class(cls):
@@ -281,10 +280,10 @@ class TestFermi3FHLObject:
 
     def test_data_python_dict(self):
         data = self.source._data_python_dict
-        assert type(data["RAJ2000"]) == float
+        assert isinstance(data["RAJ2000"], float)
         assert data["RAJ2000"] == 83.63483428955078
-        assert type(data["Flux_Band"]) == list
-        assert type(data["Flux_Band"][0]) == float
+        assert isinstance(data["Flux_Band"], list)
+        assert isinstance(data["Flux_Band"][0], float)
         assert_allclose(data["Flux_Band"][0], 5.1698894054652555e-09)
 
     @requires_dependency("uncertainties")
@@ -322,7 +321,7 @@ class TestFermi3FHLObject:
         assert str(self.cat["Crab Nebula"]) == str(self.cat[name])
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalog3FGL:
     @classmethod
     def setup_class(cls):
@@ -352,7 +351,7 @@ class TestSourceCatalog3FGL:
         assert len(selection.table) == 143
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalog1FHL:
     @classmethod
     def setup_class(cls):
@@ -372,7 +371,7 @@ class TestSourceCatalog1FHL:
         assert str(self.cat["Crab"]) == str(self.cat[name])
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalog2FHL:
     @classmethod
     def setup_class(cls):
@@ -392,7 +391,7 @@ class TestSourceCatalog2FHL:
         assert str(self.cat["Crab"]) == str(self.cat[name])
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalog3FHL:
     @classmethod
     def setup_class(cls):
