@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 from numpy.testing import assert_allclose
 import astropy.units as u
@@ -12,14 +11,14 @@ def hawc_2hwc():
     return SourceCatalog2HWC()
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalog2HWC:
     def test_source_table(self, hawc_2hwc):
         assert hawc_2hwc.name == "2hwc"
         assert len(hawc_2hwc.table) == 40
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalogObject2HWC:
     def test_data(self, hawc_2hwc):
         source = hawc_2hwc[0]
