@@ -1,9 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 from ..snrcat import SourceCatalogSNRcat
 
 
+@pytest.mark.skip(reason="see https://github.com/gammapy/gammapy/issues/2162")
 @pytest.mark.remote_data
 def test_load_catalog_snrcat(tmpdir):
     snrcat = SourceCatalogSNRcat()
