@@ -1,11 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import copy
-import operator
 import numpy as np
-from astropy.utils.decorators import lazyproperty
 import astropy.units as u
-from ..utils.fitting import Parameters, Parameter, Model
+from ..utils.fitting import Parameter, Model, Parameters
 from ..utils.scripts import make_path
 from ..maps import Map
 
@@ -13,9 +10,9 @@ __all__ = [
     "SkyModelBase",
     "SkyModels",
     "SkyModel",
-    "CompoundSkyModel",
     "SkyDiffuseCube",
     "BackgroundModel",
+    "BackgroundModels",
 ]
 
 
@@ -23,7 +20,14 @@ class SkyModelBase(Model):
     """Sky model base class"""
 
     def __add__(self, skymodel):
-        return CompoundSkyModel(self, skymodel, operator.add)
+        skymodels = [self]
+        if isinstance(skymodel, SkyModels):
+            skymodels += skymodel.skymodels
+        elif isinstance(skymodel, (SkyModel, SkyDiffuseCube)):
+            skymodels += [skymodel]
+        else:
+            raise NotImplementedError
+        return SkyModels(skymodels)
 
     def __radd__(self, model):
         return self.__add__(model)
@@ -32,7 +36,7 @@ class SkyModelBase(Model):
         return self.evaluate(lon, lat, energy)
 
 
-class SkyModels(Model):
+class SkyModels:
     """Collection of `~gammapy.cube.models.SkyModel`
 
     Parameters
@@ -42,7 +46,6 @@ class SkyModels(Model):
 
     Examples
     --------
-
     Read from an XML file::
 
         from gammapy.cube import SkyModels
@@ -50,30 +53,36 @@ class SkyModels(Model):
         sourcelib = SkyModels.read(filename)
     """
 
-    def __init__(self, skymodels):
-        self.skymodels = skymodels
+    frame = None
 
-        pars = []
-        for skymodel in skymodels:
-            for p in skymodel.parameters:
-                pars.append(p)
-        self._parameters = Parameters(pars)
+    __slots__ = ["skymodels"]
+
+    def __init__(self, skymodels):
+
+        existing_names = []
+
+        for model in skymodels:
+            if model.name in existing_names:
+                raise ValueError(
+                    "SkyModel '{}' already exists, please choose"
+                    " another name.".format(model.name)
+                )
+            existing_names.append(model.name)
+
+        self.skymodels = skymodels
 
     @property
     def parameters(self):
-        """Concatenated parameters.
-
-        Currently no way to distinguish spectral and spatial.
-        """
-        return self._parameters
-
-    @parameters.setter
-    def parameters(self, parameters):
-        idx = 0
+        parameters = []
         for skymodel in self.skymodels:
-            n_par = len(skymodel.parameters.parameters)
-            skymodel.parameters.parameters = parameters.parameters[idx : idx + n_par]
-            idx += n_par
+            for p in skymodel.parameters:
+                parameters.append(p)
+        return Parameters(parameters)
+
+    @property
+    def names(self):
+        """Sky model names"""
+        return [_.name for _ in self.skymodels]
 
     @classmethod
     def from_xml(cls, xml):
@@ -109,16 +118,50 @@ class SkyModels(Model):
         with filename.open("w") as output:
             output.write(xml)
 
-    def to_compound_model(self):
-        """Return `~gammapy.cube.models.CompoundSkyModel`"""
-        return np.sum([m for m in self.skymodels])
-
     def evaluate(self, lon, lat, energy):
         out = self.skymodels[0].evaluate(lon, lat, energy)
         for skymodel in self.skymodels[1:]:
             out += skymodel.evaluate(lon, lat, energy)
         return out
 
+    def __str__(self):
+        str_ = self.__class__.__name__ + "\n\n"
+
+        for idx, skymodel in enumerate(self.skymodels):
+            str_ += "Component {idx}: {skymodel}\n\n\t".format(
+                idx=idx, skymodel=skymodel
+            )
+            str_ += "\n\n"
+
+        if self.parameters.covariance is not None:
+            str_ += "\n\nCovariance: \n\n\t"
+            covariance = self.parameters.covariance_to_table()
+            str_ += "\n\t".join(covariance.pformat())
+        return str_
+
+    def __iadd__(self, skymodel):
+        if isinstance(skymodel, SkyModels):
+            self.skymodels += skymodel.skymodels
+        elif isinstance(skymodel, (SkyModel, SkyDiffuseCube)):
+            self.skymodels += [skymodel]
+        else:
+            raise NotImplementedError
+        return self
+
+    def __add__(self, skymodel):
+        skymodels = self.skymodels.copy()
+        if isinstance(skymodel, SkyModels):
+            skymodels += skymodel.skymodels
+        elif isinstance(skymodel, (SkyModel, SkyDiffuseCube)):
+            skymodels += [skymodel]
+        else:
+            raise NotImplementedError
+        return SkyModels(skymodels)
+
+    def __getitem__(self, item):
+        idx = self.names.index(item)
+        return self.skymodels[idx]
+
 
 class SkyModel(SkyModelBase):
     """Sky model component.
@@ -139,47 +182,49 @@ class SkyModel(SkyModelBase):
         Model identifier
     """
 
-    def __init__(self, spatial_model, spectral_model, name="SkyModel"):
+    __slots__ = ["name", "_spatial_model", "_spectral_model"]
+
+    def __init__(self, spatial_model, spectral_model, name="source"):
         self.name = name
         self._spatial_model = spatial_model
         self._spectral_model = spectral_model
-        self._parameters = Parameters(
+        parameters = (
             spatial_model.parameters.parameters + spectral_model.parameters.parameters
         )
+        super().__init__(parameters)
 
     @property
     def spatial_model(self):
         """`~gammapy.image.models.SkySpatialModel`"""
-        # propagate sub-covariance
-        if self.parameters.covariance is not None:
-            idx = len(self._spatial_model.parameters.parameters)
-            self._spatial_model.parameters.covariance = self.parameters.covariance[
-                :idx, :idx
-            ]
         return self._spatial_model
 
     @property
     def spectral_model(self):
         """`~gammapy.spectrum.models.SpectralModel`"""
-        # propagate sub-covariance
-        if self.parameters.covariance is not None:
-            idx = len(self._spatial_model.parameters.parameters)
-            self._spectral_model.parameters.covariance = self.parameters.covariance[
-                idx:, idx:
-            ]
         return self._spectral_model
 
-    @property
-    def parameters(self):
-        """Parameters (`~gammapy.utils.modeling.Parameters`)"""
-        return self._parameters
+    @spectral_model.setter
+    def spectral_model(self, model):
+        """`~gammapy.spectrum.models.SpectralModel`"""
+        self._spectral_model = model
+        self._parameters = Parameters(
+            self.spatial_model.parameters.parameters
+            + self.spectral_model.parameters.parameters
+        )
 
-    @parameters.setter
-    def parameters(self, parameters):
-        self._parameters = parameters
-        idx = len(self.spatial_model.parameters.parameters)
-        self._spatial_model.parameters.parameters = parameters.parameters[:idx]
-        self._spectral_model.parameters.parameters = parameters.parameters[idx:]
+    @property
+    def position(self):
+        """`~astropy.coordinates.SkyCoord`"""
+        return self.spatial_model.position
+
+    @property
+    def evaluation_radius(self):
+        """`~astropy.coordinates.Angle`"""
+        return self.spatial_model.evaluation_radius
+
+    @property
+    def frame(self):
+        return self.spatial_model.frame
 
     def __repr__(self):
         fmt = "{}(spatial_model={!r}, spectral_model={!r})"
@@ -209,72 +254,14 @@ class SkyModel(SkyModelBase):
         """
         val_spatial = self.spatial_model(lon, lat)  # pylint:disable=not-callable
         val_spectral = self.spectral_model(energy)  # pylint:disable=not-callable
-        val = val_spatial * val_spectral
-        # TODO: shall remove hard coded return units? If really needed users can
-        # always do this themselves. For fitting this also adds a performance penalty...
-        return val.to("cm-2 s-1 TeV-1 deg-2")
+        return val_spatial * val_spectral
 
-
-class CompoundSkyModel(SkyModelBase):
-    """Represents the algebraic combination of two
-    `~gammapy.cube.models.SkyModel`
-
-    Parameters
-    ----------
-    model1, model2 : `SkyModel`
-        Two sky models
-    operator : callable
-        Binary operator to combine the models
-    """
-
-    def __init__(self, model1, model2, operator):
-        self.model1 = model1
-        self.model2 = model2
-        self.operator = operator
-        self._parameters = Parameters(
-            self.model1.parameters.parameters + self.model2.parameters.parameters
-        )
-
-    @property
-    def parameters(self):
-        """Parameters (`~gammapy.utils.modeling.Parameters`)"""
-        return self._parameters
-
-    @parameters.setter
-    def parameters(self, parameters):
-        self._parameters = parameters
-        idx = len(self.model1.parameters.parameters)
-        self.model1.parameters.parameters = parameters.parameters[:idx]
-        self.model2.parameters.parameters = parameters.parameters[idx:]
-
-    def __str__(self):
-        ss = self.__class__.__name__
-        ss += "\n    Component 1 : {}".format(self.model1)
-        ss += "\n    Component 2 : {}".format(self.model2)
-        ss += "\n    Operator : {}".format(self.operator)
-        return ss
-
-    def evaluate(self, lon, lat, energy):
-        """Evaluate the compound model at given points.
-
-        Return differential surface brightness cube.
-        At the moment in units: ``cm-2 s-1 TeV-1 deg-2``
-
-        Parameters
-        ----------
-        lon, lat : `~astropy.units.Quantity`
-            Spatial coordinates
-        energy : `~astropy.units.Quantity`
-            Energy coordinate
-
-        Returns
-        -------
-        value : `~astropy.units.Quantity`
-            Model value at the given point.
-        """
-        val1 = self.model1.evaluate(lon, lat, energy)
-        val2 = self.model2.evaluate(lon, lat, energy)
-        return self.operator(val1, val2)
+    def copy(self, **kwargs):
+        """Copy SkyModel"""
+        kwargs.setdefault("spatial_model", self.spatial_model.copy())
+        kwargs.setdefault("spectral_model", self.spectral_model.copy())
+        kwargs.setdefault("name", self.name + "-copy")
+        return self.__class__(**kwargs)
 
 
 class SkyDiffuseCube(SkyModelBase):
@@ -292,19 +279,22 @@ class SkyDiffuseCube(SkyModelBase):
     meta : dict, optional
         Meta information, meta['filename'] will be used for serialization
     interp_kwargs : dict
-        Interpolation keyword arguments passed to `Map.interp_by_coord()`.
+        Interpolation keyword arguments passed to `gammapy.maps.Map.interp_by_coord`.
         Default arguments are {'interp': 'linear', 'fill_value': 0}.
 
     """
 
-    def __init__(self, map, norm=1, meta=None, interp_kwargs=None):
+    __slots__ = ["map", "norm", "meta", "_interp_kwargs"]
+
+    def __init__(self, map, norm=1, meta=None, interp_kwargs=None, name="diffuse"):
+        self.name = name
         axis = map.geom.get_axis_by_name("energy")
 
         if axis.node_type != "center":
             raise ValueError('Need a map with energy axis node_type="center"')
 
         self.map = map
-        self.parameters = Parameters([Parameter("norm", norm)])
+        self.norm = Parameter("norm", norm)
         self.meta = {} if meta is None else meta
 
         interp_kwargs = {} if interp_kwargs is None else interp_kwargs
@@ -312,6 +302,8 @@ class SkyDiffuseCube(SkyModelBase):
         interp_kwargs.setdefault("fill_value", 0)
         self._interp_kwargs = interp_kwargs
 
+        super().__init__([self.norm])
+
     @classmethod
     def read(cls, filename, **kwargs):
         """Read map from FITS file.
@@ -343,9 +335,26 @@ class SkyDiffuseCube(SkyModelBase):
         """A shallow copy"""
         return copy.copy(self)
 
+    @property
+    def position(self):
+        """`~astropy.coordinates.SkyCoord`"""
+        return self.map.geom.center_skydir
+
+    @property
+    def evaluation_radius(self):
+        """`~astropy.coordinates.Angle`"""
+        radius = np.max(self.map.geom.width) / 2.0
+        return radius
+
+    @property
+    def frame(self):
+        return self.position.frame.name
+
 
 class BackgroundModel(Model):
-    """Background model
+    """Background model.
+
+    Create a new map by a tilt and normalisation on the available map
 
     Parameters
     ----------
@@ -359,32 +368,116 @@ class BackgroundModel(Model):
         Reference energy of the tilt.
     """
 
-    def __init__(self, background, norm=1, tilt=0, reference="1 TeV"):
+    __slots__ = ["map", "norm", "tilt", "reference"]
 
+    def __init__(self, background, norm=1, tilt=0, reference="1 TeV"):
         axis = background.geom.get_axis_by_name("energy")
         if axis.node_type != "edges":
             raise ValueError('Need an integrated map, energy axis node_type="edges"')
 
         self.map = background
-        self.parameters = Parameters(
-            [
-                Parameter("norm", norm, unit=""),
-                Parameter("tilt", tilt, unit="", frozen=True),
-                Parameter("reference", reference, frozen=True),
-            ]
-        )
+        self.norm = Parameter("norm", norm, unit="", min=0)
+        self.tilt = Parameter("tilt", tilt, unit="", frozen=True)
+        self.reference = Parameter("reference", reference, frozen=True)
 
-    @lazyproperty
+        super().__init__([self.norm, self.tilt, self.reference])
+
+    @property
     def energy_center(self):
         """True energy axis bin centers (`~astropy.units.Quantity`)"""
         energy_axis = self.map.geom.get_axis_by_name("energy")
-        energy = energy_axis.center * energy_axis.unit
+        energy = energy_axis.center
         return energy[:, np.newaxis, np.newaxis]
 
     def evaluate(self):
-        """Evaluate background model"""
+        """Evaluate background model.
+
+        Returns
+        -------
+        background_map : `~gammapy.maps.Map`
+            Background evaluated on the Map
+        """
         norm = self.parameters["norm"].value
         tilt = self.parameters["tilt"].value
         reference = self.parameters["reference"].quantity
         tilt_factor = np.power((self.energy_center / reference).to(""), -tilt)
-        return u.Quantity(norm * self.map.data * tilt_factor, self.map.unit, copy=False)
+        back_values = norm * self.map.data * tilt_factor.value
+        return self.map.copy(data=back_values)
+
+    @classmethod
+    def from_skymodel(cls, skymodel, exposure, edisp=None, psf=None, **kwargs):
+        """Create background model from sky model by applying IRFs.
+
+        Typically used for diffuse Galactic or constant emission models.
+
+        Parameters
+        ----------
+        skymodel : `~gammapy.cube.models.SkyModel` or `~gammapy.cube.models.SkyDiffuseCube`
+            Sky model
+        exposure : `~gammapy.maps.Map`
+            Exposure map
+        edisp : `~gammapy.irf.EnergyDispersion`
+            Energy dispersion
+        psf : `~gammapy.cube.PSFKernel`
+            PSF kernel
+        """
+        from .fit import MapEvaluator
+
+        evaluator = MapEvaluator(
+            model=skymodel, exposure=exposure, edisp=edisp, psf=psf
+        )
+        background = evaluator.compute_npred()
+        return cls(background=background, **kwargs)
+
+    def __add__(self, model):
+        models = [self]
+        if isinstance(model, BackgroundModels):
+            models += model.models
+        elif isinstance(model, BackgroundModel):
+            models += [model]
+        else:
+            raise NotImplementedError
+        return BackgroundModels(models)
+
+
+class BackgroundModels(Model):
+    """Background models.
+
+    Parameters
+    ----------
+    models : list of `BackgroundModel`
+        List of background models.
+    """
+
+    __slots__ = ["models", "_parameters"]
+
+    def __init__(self, models):
+        self.models = models
+        parameters = []
+        for model in models:
+            for p in model.parameters:
+                parameters.append(p)
+        super().__init__(parameters)
+
+    def evaluate(self):
+        """Evaluate background models."""
+        for idx, model in enumerate(self.models):
+            if idx == 0:
+                vals = model.evaluate()
+            else:
+                vals += model.evaluate()
+        return vals
+
+    def __iadd__(self, model):
+        if isinstance(model, BackgroundModels):
+            self.models += model.models
+        elif isinstance(model, BackgroundModel):
+            self.models += [model]
+        else:
+            raise NotImplementedError
+        return self
+
+    def __add__(self, model):
+        model_ = self.copy()
+        model_ += model
+        return model_
