@@ -1,8 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 import astropy.units as u
 from astropy.coordinates import Angle
+import astropy.io.fits as fits
 from ..irf import EnergyDependentTablePSF
 from ..maps import Map
 from ..cube import PSFKernel
@@ -10,7 +10,7 @@ from ..cube import PSFKernel
 __all__ = ["make_psf_map", "PSFMap"]
 
 
-def make_psf_map(psf, pointing, geom, max_offset):
+def make_psf_map(psf, pointing, geom, max_offset, exposure_map=None):
     """Make a psf map for a single observation
 
     Expected axes : rad and true energy in this specific order
@@ -27,14 +27,16 @@ def make_psf_map(psf, pointing, geom, max_offset):
         rad and true energy axes should be given in this specific order.
     max_offset : `~astropy.coordinates.Angle`
         maximum offset w.r.t. fov center
+    exposure_map : `~gammapy.maps.Map`, optional
+        the associated exposure map.
+        default is None
 
     Returns
     -------
     psfmap : `~gammapy.cube.PSFMap`
         the resulting PSF map
     """
-    energy_axis = geom.get_axis_by_name("energy")
-    energy = energy_axis.center * energy_axis.unit
+    energy = geom.get_axis_by_name("energy").center
 
     rad_axis = geom.get_axis_by_name("theta")
     rad = Angle(rad_axis.center, unit=rad_axis.unit)
@@ -49,14 +51,15 @@ def make_psf_map(psf, pointing, geom, max_offset):
     # Re-order axes to be consistent with expected geometry
     psf_values = np.transpose(psf_values, axes=(2, 0, 1))
 
+    # TODO: this probably does not ensure that probability is properly normalized in the PSFMap
     # Create Map and fill relevant entries
     psfmap = Map.from_geom(geom, unit="sr-1")
     psfmap.data[:, :, valid[0], valid[1]] += psf_values.to_value(psfmap.unit)
 
-    return PSFMap(psfmap)
+    return PSFMap(psfmap, exposure_map)
 
 
-class PSFMap(object):
+class PSFMap:
     """Class containing the Map of PSFs and allowing to interact with it.
 
     Parameters
@@ -64,6 +67,8 @@ class PSFMap(object):
     psf_map : `~gammapy.maps.Map`
         the input PSF Map. Should be a Map with 2 non spatial axes.
         rad and true energy axes should be given in this specific order.
+    exposure_map : `~gammapy.maps.Map`
+        Associated exposure map. Needs to have a consistent map geometry.
 
     Examples
     --------
@@ -73,8 +78,8 @@ class PSFMap(object):
         from astropy import units as u
         from astropy.coordinates import SkyCoord
         from gammapy.maps import Map, WcsGeom, MapAxis
-        from gammapy.irf import EnergyDependentMultiGaussPSF
-        from gammapy.cube import make_psf_map, PSFMap
+        from gammapy.irf import EnergyDependentMultiGaussPSF, EffectiveAreaTable2D
+        from gammapy.cube import make_psf_map, PSFMap, make_map_exposure_true_energy
 
         # Define energy axis. Note that the name is fixed.
         energy_axis = MapAxis.from_edges(np.logspace(-1., 1., 4), unit='TeV', name='energy')
@@ -93,9 +98,14 @@ class PSFMap(object):
         filename = '$GAMMAPY_DATA/cta-1dc/caldb/data/cta/1dc/bcf/South_z20_50h/irf_file.fits'
         psf = EnergyDependentMultiGaussPSF.read(filename, hdu='POINT SPREAD FUNCTION')
         psf3d = psf.to_psf3d(rads)
+        aeff2d = EffectiveAreaTable2D.read(filename, hdu='EFFECTIVE AREA')
+
+        # Create the exposure map
+        exposure_geom = geom.to_image().to_cube([energy_axis])
+        exposure_map = make_map_exposure_true_energy(pointing, "1 h", aeff2d, exposure_geom)
 
         # create the PSFMap for the specified pointing
-        psf_map = make_psf_map(psf3d, pointing, geom, max_offset)
+        psf_map = make_psf_map(psf3d, pointing, geom, max_offset, exposure_map)
 
         # Get an EnergyDependentTablePSF at any position in the image
         psf_table = psf_map.get_energy_dependent_table_psf(SkyCoord(2., 2.5, unit='deg'))
@@ -104,47 +114,100 @@ class PSFMap(object):
         psf_map.write('psf_map.fits')
     """
 
-    def __init__(self, psf_map):
+    def __init__(self, psf_map, exposure_map=None):
         if psf_map.geom.axes[1].name.upper() != "ENERGY":
             raise ValueError("Incorrect energy axis position in input Map")
 
         if psf_map.geom.axes[0].name.upper() != "THETA":
             raise ValueError("Incorrect theta axis position in input Map")
 
-        self._psf_map = psf_map
+        self.psf_map = psf_map
 
-    @property
-    def psf_map(self):
-        """the PSFMap itself (`~gammapy.maps.Map`)"""
-        return self._psf_map
+        if exposure_map is not None:
+            # First adapt geometry, keep only energy axis
+            expected_geom = psf_map.geom.to_image().to_cube([psf_map.geom.axes[1]])
+            if exposure_map.geom != expected_geom:
+                raise ValueError("PSFMap and exposure_map have inconsistent geometries")
 
-    @property
-    def data(self):
-        """the PSFMap data"""
-        return self._psf_map.data
+        self.exposure_map = exposure_map
 
-    @property
-    def quantity(self):
-        """the PSFMap data as a quantity"""
-        return self._psf_map.quantity
+    @classmethod
+    def from_hdulist(
+        cls,
+        hdulist,
+        psf_hdu="PSFMAP",
+        psf_hdubands="BANDSPSF",
+        exposure_hdu="EXPMAP",
+        exposure_hdubands="BANDSEXP",
+    ):
+        """Convert to `~astropy.io.fits.HDUList`.
 
-    @property
-    def geom(self):
-        """The PSFMap MapGeom object"""
-        return self._psf_map.geom
+        Parameters
+        ----------
+        psf_hdu : str
+            Name or index of the HDU with the psf_map data.
+        psf_hdubands : str
+            Name or index of the HDU with the psf_map BANDS table.
+        exposure_hdu : str
+            Name or index of the HDU with the exposure_map data.
+        exposure_hdubands : str
+            Name or index of the HDU with the exposure_map BANDS table.
+        """
+        psf_map = Map.from_hdulist(hdulist, psf_hdu, psf_hdubands, "auto")
+        if exposure_hdu in hdulist:
+            exposure_map = Map.from_hdulist(
+                hdulist, exposure_hdu, exposure_hdubands, "auto"
+            )
+        else:
+            exposure_map = None
+
+        return cls(psf_map, exposure_map)
 
     @classmethod
     def read(cls, filename, **kwargs):
         """Read a psf_map from file and create a PSFMap object"""
-        psfmap = Map.read(filename, **kwargs)
-        return cls(psfmap)
+        with fits.open(filename, memmap=False) as hdulist:
+            return cls.from_hdulist(hdulist, **kwargs)
 
-    def write(self, *args, **kwargs):
-        """Write the Map object containing the PSF Library map."""
-        self._psf_map.write(*args, **kwargs)
+    def to_hdulist(
+        self,
+        psf_hdu="PSFMAP",
+        psf_hdubands="BANDSPSF",
+        exposure_hdu="EXPMAP",
+        exposure_hdubands="BANDSEXP",
+    ):
+        """Convert to `~astropy.io.fits.HDUList`.
+
+        Parameters
+        ----------
+        psf_hdu : str
+            Name or index of the HDU with the psf_map data.
+        psf_hdubands : str
+            Name or index of the HDU with the psf_map BANDS table.
+        exposure_hdu : str
+            Name or index of the HDU with the exposure_map data.
+        exposure_hdubands : str
+            Name or index of the HDU with the exposure_map BANDS table.
+
+        Returns
+        -------
+        hdu_list : `~astropy.io.fits.HDUList`
+        """
+        hdulist = self.psf_map.to_hdulist(hdu=psf_hdu, hdu_bands=psf_hdubands)
+        if self.exposure_map is not None:
+            new_hdulist = self.exposure_map.to_hdulist(
+                hdu=exposure_hdu, hdu_bands=exposure_hdubands
+            )
+            hdulist.extend(new_hdulist[1:])
+        return hdulist
+
+    def write(self, filename, overwrite=False, **kwargs):
+        """Write to fits"""
+        hdulist = self.to_hdulist(**kwargs)
+        hdulist.writeto(filename, overwrite=overwrite)
 
     def get_energy_dependent_table_psf(self, position):
-        """ Returns EnergyDependentTable PSF at a given position
+        """Get energy-dependent PSF at a given position.
 
         Parameters
         ----------
@@ -162,8 +225,8 @@ class PSFMap(object):
             )
 
         # axes ordering fixed. Could be changed.
-        pix_ener = np.arange(self.geom.axes[1].nbin)
-        pix_rad = np.arange(self.geom.axes[0].nbin)
+        pix_ener = np.arange(self.psf_map.geom.axes[1].nbin)
+        pix_rad = np.arange(self.psf_map.geom.axes[0].nbin)
 
         # Convert position to pixels
         pix_lon, pix_lat = self.psf_map.geom.to_image().coord_to_pix(position)
@@ -176,8 +239,8 @@ class PSFMap(object):
             self.psf_map.interp_by_pix(pix) * u.Unit(self.psf_map.unit)
         )
 
-        energies = self.psf_map.geom.axes[1].center * self.psf_map.geom.axes[1].unit
-        rad = self.psf_map.geom.axes[0].center * self.psf_map.geom.axes[0].unit
+        energies = self.psf_map.geom.axes[1].center
+        rad = self.psf_map.geom.axes[0].center
 
         # Beware. Need to revert rad and energies to follow the TablePSF scheme.
         return EnergyDependentTablePSF(energy=energies, rad=rad, psf_value=psf_values.T)
@@ -221,8 +284,8 @@ class PSFMap(object):
         containment_radius_map : `~gammapy.maps.Map`
             Containment radius map
         """
-        coords = self.geom.to_image().get_coord().skycoord.flatten()
-        m = Map.from_geom(self.geom.to_image(), unit="deg")
+        coords = self.psf_map.geom.to_image().get_coord().skycoord.flatten()
+        m = Map.from_geom(self.psf_map.geom.to_image(), unit="deg")
 
         for coord in coords:
             psf_table = self.get_energy_dependent_table_psf(coord)
@@ -230,3 +293,52 @@ class PSFMap(object):
             m.fill_by_coord(coord, containment_radius)
 
         return m
+
+    def stack(self, other):
+        """Stack PSFMap with another one.
+
+        The other PSFMap is projected on the current PSFMap geometry.
+
+        Parameters
+        ----------
+        other : `~gammapy.cube.PSFMap`
+            the psfmap to be stacked with this one.
+
+        Returns
+        -------
+        new : `~gammapy.cube.PSFMap`
+            the stacked psfmap
+        """
+        if self.exposure_map is None or other.exposure_map is None:
+            raise ValueError("Missing exposure map for PSFMap.stack")
+
+        # Reproject other exposure
+        exposure_coord = self.exposure_map.geom.get_coord()
+        reproj_exposure = Map.from_geom(
+            self.exposure_map.geom, unit=self.exposure_map.unit
+        )
+        reproj_exposure.fill_by_coord(
+            exposure_coord, other.exposure_map.get_by_coord(exposure_coord)
+        )
+
+        # Reproject other psfmap using same geom
+        psfmap_coord = self.psf_map.geom.get_coord()
+        reproj_psfmap = Map.from_geom(self.psf_map.geom, unit=self.psf_map.unit)
+        reproj_psfmap.fill_by_coord(
+            psfmap_coord, other.psf_map.get_by_coord(psfmap_coord)
+        )
+
+        exposure = self.exposure_map.quantity[:, np.newaxis, :, :]
+        stacked_psf_quantity = self.psf_map.quantity * exposure
+
+        other_exposure = reproj_exposure.quantity[:, np.newaxis, :, :]
+        stacked_psf_quantity += reproj_psfmap.quantity * other_exposure
+
+        total_exposure = exposure + other_exposure
+        stacked_psf_quantity /= total_exposure
+
+        reproj_psfmap.quantity = stacked_psf_quantity
+        # We need to remove the extra axis in the total exposure
+        reproj_exposure.quantity = total_exposure[:, 0, :, :]
+
+        return PSFMap(reproj_psfmap, reproj_exposure)
