@@ -1,13 +1,15 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
+import numpy as np
 from numpy.testing import assert_allclose
 import astropy.units as u
 from astropy.coordinates import SkyCoord
+from regions import CircleSkyRegion
 from ...utils.testing import requires_data
 from ...data import DataStore
 from ...maps import WcsGeom, MapAxis, Map
-from ..make import MapMaker
+from ..make import MapMaker, MapMakerRing
+from ...background import RingBackgroundEstimator
 
 
 @pytest.fixture(scope="session")
@@ -29,7 +31,7 @@ def geom(ebounds):
     )
 
 
-@requires_data("gammapy-data")
+@requires_data()
 @pytest.mark.parametrize(
     "pars",
     [
@@ -40,7 +42,7 @@ def geom(ebounds):
             "counts": 34366,
             "exposure": 3.99815e11,
             "exposure_image": 7.921993e10,
-            "background": 27984.78,
+            "background": 27986.945,
         },
         {
             # Test single energy bin
@@ -49,7 +51,7 @@ def geom(ebounds):
             "counts": 34366,
             "exposure": 1.16866e11,
             "exposure_image": 1.16866e11,
-            "background": 30418.951,
+            "background": 30422.17,
         },
         {
             # Test single energy bin with exclusion mask
@@ -59,7 +61,7 @@ def geom(ebounds):
             "counts": 34366,
             "exposure": 1.16866e11,
             "exposure_image": 1.16866e11,
-            "background": 30418.951,
+            "background": 30422.17,
         },
         {
             # Test for different e_true and e_reco bins
@@ -68,13 +70,19 @@ def geom(ebounds):
             "counts": 34366,
             "exposure": 5.971096e11,
             "exposure_image": 6.492968e10,
-            "background": 27984.78,
+            "background": 28758.129,
+            "background_oversampling": 2,
         },
     ],
 )
 @pytest.mark.parametrize("keepdims", [True, False])
 def test_map_maker(pars, observations, keepdims):
-    maker = MapMaker(geom=pars["geom"], geom_true=pars["geom_true"], offset_max="2 deg")
+    maker = MapMaker(
+        geom=pars["geom"],
+        geom_true=pars["geom_true"],
+        offset_max="2 deg",
+        background_oversampling=pars.get("background_oversampling"),
+    )
 
     maps = maker.run(observations)
 
@@ -90,7 +98,7 @@ def test_map_maker(pars, observations, keepdims):
     assert background.unit == ""
     assert_allclose(background.data.sum(), pars["background"], rtol=1e-5)
 
-    images = maker.make_images(keepdims=keepdims)
+    images = maker.run_images(keepdims=keepdims)
 
     counts = images["counts"]
     assert counts.unit == ""
@@ -103,3 +111,28 @@ def test_map_maker(pars, observations, keepdims):
     background = images["background"]
     assert background.unit == ""
     assert_allclose(background.data.sum(), pars["background"], rtol=1e-5)
+
+
+@requires_data()
+def test_map_maker_ring(observations):
+    ring_bkg = RingBackgroundEstimator(r_in="0.5 deg", width="0.4 deg")
+    geomd = geom(ebounds=[0.1, 1, 10])
+
+    mask = Map.from_geom(geomd)
+
+    regions = CircleSkyRegion(
+        SkyCoord(0, 0, unit="deg", frame="galactic"), radius=0.5 * u.deg
+    )
+    mask.data = mask.geom.region_mask([regions], inside=False)
+
+    maker = MapMakerRing(geomd, 2.0 * u.deg, mask, ring_bkg)
+
+    maps = maker.run(observations)
+    assert_allclose(np.nansum(maps["on"].data), 21981, rtol=1e-2)
+    assert_allclose(np.nansum(maps["exposure_off"].data), 8310.035, rtol=1e-2)
+    assert maps["on"].geom.is_image is False
+
+    images = maker.run_images(observations)
+    assert_allclose(np.nansum(images["on"].data), 21981, rtol=1e-2)
+    assert_allclose(np.nansum(images["exposure_off"].data), 109751.45, rtol=1e-2)
+    assert images["on"].geom.is_image is True
