@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
@@ -8,11 +7,11 @@ from ...utils.testing import requires_data
 from ...maps import MapAxis, WcsGeom, Map
 from ...irf.energy_dispersion import EnergyDispersion
 from ...cube.psf_kernel import PSFKernel
-from ...cube.models import SkyDiffuseCube
+from ...cube.models import SkyDiffuseCube, BackgroundModel, BackgroundModels
 from ...image.models import SkyGaussian
 from ...spectrum.models import PowerLaw
 from ..fit import MapEvaluator
-from ..models import SkyModel, SkyModels, CompoundSkyModel
+from ..models import SkyModel, SkyModels
 
 
 @pytest.fixture(scope="session")
@@ -21,13 +20,15 @@ def sky_model():
     spectral_model = PowerLaw(
         index=2, amplitude="1e-11 cm-2 s-1 TeV-1", reference="1 TeV"
     )
-    return SkyModel(spatial_model, spectral_model)
+    return SkyModel(spatial_model, spectral_model, name="source-1")
 
 
 @pytest.fixture(scope="session")
 def diffuse_model():
     axis = MapAxis.from_nodes([0.1, 100], name="energy", unit="TeV", interp="log")
-    m = Map.create(npix=(4, 3), binsz=2, axes=[axis], unit="cm-2 s-1 MeV-1 sr-1")
+    m = Map.create(
+        npix=(4, 3), binsz=2, axes=[axis], unit="cm-2 s-1 MeV-1 sr-1", coordsys="GAL"
+    )
     m.data += 42
     return SkyDiffuseCube(m)
 
@@ -73,30 +74,78 @@ def psf(geom_true):
 
 
 @pytest.fixture(scope="session")
-def evaluator(sky_model, exposure, background, psf, edisp):
-    return MapEvaluator(sky_model, exposure, background, psf=psf, edisp=edisp)
+def evaluator(sky_model, exposure, psf, edisp):
+    return MapEvaluator(sky_model, exposure, psf=psf, edisp=edisp)
 
 
 @pytest.fixture(scope="session")
-def diffuse_evaluator(diffuse_model, exposure, background, psf, edisp):
-    return MapEvaluator(diffuse_model, exposure, background, psf=psf, edisp=edisp)
+def diffuse_evaluator(diffuse_model, exposure, psf, edisp):
+    return MapEvaluator(diffuse_model, exposure, psf=psf, edisp=edisp)
+
+
+@pytest.fixture(scope="session")
+def sky_models(sky_model):
+    sky_model_2 = sky_model.copy(name="source-2")
+    sky_model_3 = sky_model.copy(name="source-3")
+    return SkyModels([sky_model_2, sky_model_3])
+
+
+@pytest.fixture(scope="session")
+def sky_models_2(sky_model):
+    sky_model_4 = sky_model.copy(name="source-4")
+    sky_model_5 = sky_model.copy(name="source-5")
+    return SkyModels([sky_model_4, sky_model_5])
+
+
+def test_skymodel_addition(sky_model, sky_models, sky_models_2, diffuse_model):
+    result = sky_model + sky_model.copy()
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 2
+
+    result = sky_model + sky_models
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 3
+
+    result = sky_models + sky_model
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 3
+
+    result = sky_models + diffuse_model
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 3
+
+    result = sky_models + sky_models_2
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 4
+
+    result = sky_model + sky_models
+    assert isinstance(result, SkyModels)
+    assert len(result.skymodels) == 3
+
+
+def test_background_model(background):
+    bkg1 = BackgroundModel(background, norm=2.0).evaluate()
+    assert_allclose(bkg1.data[0][0][0], background.data[0][0][0] * 2.0, rtol=1e-3)
+    assert_allclose(bkg1.data.sum(), background.data.sum() * 2.0, rtol=1e-3)
+
+    bkg2 = BackgroundModel(
+        background, norm=2.0, tilt=0.2, reference="1000 GeV"
+    ).evaluate()
+    assert_allclose(bkg2.data[0][0][0], 2.254e-07, rtol=1e-3)
+    assert_allclose(bkg2.data.sum(), 7.352e-06, rtol=1e-3)
+
+
+def test_background_models(background):
+    bkg_1 = BackgroundModel(background, norm=1.0)
+    bkg_2 = BackgroundModel(background, norm=2.0)
+    models = BackgroundModels([bkg_1, bkg_2])
+    bkg_eval = models.evaluate()
+    assert_allclose(3 * bkg_1.map.data[0][0][0], bkg_eval.data[0][0][0])
 
 
 class TestSkyModels:
-    def setup(self):
-        self.sky_models = SkyModels([sky_model(), sky_model()])
-
-    def test_to_compound_model(self):
-        sky_models = self.sky_models
-        model = sky_models.to_compound_model()
-        assert isinstance(model, CompoundSkyModel)
-        pars = model.parameters.parameters
-        assert len(pars) == 12
-        assert pars[0].name == "lon_0"
-        assert pars[-1].name == "reference"
-
-    def test_parameters(self):
-        sky_models = self.sky_models
+    @staticmethod
+    def test_parameters(sky_models):
         parnames = ["lon_0", "lat_0", "sigma", "index", "amplitude", "reference"] * 2
         assert sky_models.parameters.names == parnames
 
@@ -105,22 +154,34 @@ class TestSkyModels:
         p2 = sky_models.skymodels[0].parameters["lon_0"]
         assert p1 is p2
 
-        # Check that parameter assignment works
-        assert sky_models.parameters.parameters[-1].value == 1
-        sky_models.parameters = sky_models.parameters.copy()
-        assert sky_models.parameters.parameters[-1].value == 1
-
-    def test_evaluate(self):
-        sky_models = self.sky_models
+    @staticmethod
+    def test_evaluate(sky_models):
         lon = 3 * u.deg * np.ones(shape=(3, 4))
         lat = 4 * u.deg * np.ones(shape=(3, 4))
         energy = [1, 1, 1, 1, 1] * u.TeV
 
         q = sky_models.evaluate(lon, lat, energy[:, np.newaxis, np.newaxis])
 
-        assert q.unit == "cm-2 s-1 TeV-1 deg-2"
+        assert q.unit == "cm-2 s-1 TeV-1 sr-1"
         assert q.shape == (5, 3, 4)
-        assert_allclose(q.value, 3.536776513153229e-13)
+        assert_allclose(q.to_value("cm-2 s-1 TeV-1 deg-2"), 3.53758465e-13)
+
+    @staticmethod
+    def test_str(sky_models):
+        assert "Component 0" in str(sky_models)
+        assert "Component 1" in str(sky_models)
+
+    @staticmethod
+    def test_get_item(sky_models):
+        model = sky_models["source-2"]
+        assert model.name == "source-2"
+
+        model = sky_models["source-3"]
+        assert model.name == "source-3"
+
+        with pytest.raises(ValueError) as error:
+            sky_models["spam"]
+            assert "spam" in error.message
 
 
 class TestSkyModel:
@@ -151,9 +212,9 @@ class TestSkyModel:
 
         q = sky_model.evaluate(lon, lat, energy)
 
-        assert q.unit == "cm-2 s-1 TeV-1 deg-2"
+        assert q.unit == "cm-2 s-1 TeV-1 sr-1"
         assert np.isscalar(q.value)
-        assert_allclose(q.value, 1.76838826e-13)
+        assert_allclose(q.to_value("cm-2 s-1 TeV-1 deg-2"), 1.76879232e-13)
 
     @staticmethod
     def test_evaluate_array(sky_model):
@@ -164,42 +225,7 @@ class TestSkyModel:
         q = sky_model.evaluate(lon, lat, energy[:, np.newaxis, np.newaxis])
 
         assert q.shape == (5, 3, 4)
-        assert_allclose(q.value, 1.76838826e-13)
-
-
-class TestCompoundSkyModel:
-    @staticmethod
-    @pytest.fixture()
-    def compound_model(sky_model):
-        return sky_model + sky_model
-
-    @staticmethod
-    def test_parameters(compound_model):
-        parnames = ["lon_0", "lat_0", "sigma", "index", "amplitude", "reference"] * 2
-        assert compound_model.parameters.names == parnames
-
-        # Check that model parameters are references to the parts
-        assert (
-            compound_model.parameters["lon_0"]
-            is compound_model.model1.parameters["lon_0"]
-        )
-
-        # Check that parameter assignment works
-        assert compound_model.parameters.parameters[-1].value == 1
-        compound_model.parameters = compound_model.parameters.copy()
-        assert compound_model.parameters.parameters[-1].value == 1
-
-    @staticmethod
-    def test_evaluate(compound_model):
-        lon = 3 * u.deg * np.ones(shape=(3, 4))
-        lat = 4 * u.deg * np.ones(shape=(3, 4))
-        energy = [1, 1, 1, 1, 1] * u.TeV
-
-        q = compound_model.evaluate(lon, lat, energy[:, np.newaxis, np.newaxis])
-
-        assert q.unit == "cm-2 s-1 TeV-1 deg-2"
-        assert q.shape == (5, 3, 4)
-        assert_allclose(q.value, 3.536776513153229e-13)
+        assert_allclose(q.to_value("cm-2 s-1 TeV-1 deg-2"), 1.76879232e-13)
 
 
 class TestSkyDiffuseCube:
@@ -231,10 +257,10 @@ class TestSkyDiffuseCube:
         assert_allclose(q.value.mean(), 42)
 
     @staticmethod
-    @requires_data("gammapy-extra")
+    @requires_data()
     def test_read():
         model = SkyDiffuseCube.read(
-            "$GAMMAPY_EXTRA/test_datasets/unbundled/fermi/gll_iem_v02_cutout.fits"
+            "$GAMMAPY_DATA/tests/unbundled/fermi/gll_iem_v02_cutout.fits"
         )
         assert model.map.unit == "cm-2 s-1 MeV-1 sr-1"
 
@@ -244,6 +270,16 @@ class TestSkyDiffuseCube:
         assert val.shape == (1,)
         assert_allclose(val.value, 1.396424e-12, rtol=1e-5)
 
+    @staticmethod
+    def test_evaluation_radius(diffuse_model):
+        radius = diffuse_model.evaluation_radius
+        assert radius.unit == "deg"
+        assert_allclose(radius.value, 4)
+
+    @staticmethod
+    def test_frame(diffuse_model):
+        assert diffuse_model.frame == "galactic"
+
 
 class TestSkyDiffuseCubeMapEvaluator:
     @staticmethod
@@ -260,7 +296,7 @@ class TestSkyDiffuseCubeMapEvaluator:
         assert out.shape == (3, 4, 5)
         out = out.to("cm-2 s-1")
         assert_allclose(out.value.sum(), 633263.444803, rtol=1e-5)
-        assert_allclose(out.value[0, 0, 0], 1164.578565, rtol=1e-5)
+        assert_allclose(out.value[0, 0, 0], 1164.656176, rtol=1e-5)
 
     @staticmethod
     def test_apply_psf(diffuse_evaluator):
@@ -269,7 +305,7 @@ class TestSkyDiffuseCubeMapEvaluator:
         out = diffuse_evaluator.apply_psf(npred)
         assert out.data.shape == (3, 4, 5)
         assert_allclose(out.data.sum(), 1.106404e12, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 5.586252e08, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 5.586508e08, rtol=1e-5)
 
     @staticmethod
     def test_apply_edisp(diffuse_evaluator):
@@ -278,14 +314,14 @@ class TestSkyDiffuseCubeMapEvaluator:
         out = diffuse_evaluator.apply_edisp(npred)
         assert out.data.shape == (2, 4, 5)
         assert_allclose(out.data.sum(), 1.606345e12, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 1.164579e09, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 1.164656e09, rtol=1e-5)
 
     @staticmethod
     def test_compute_npred(diffuse_evaluator):
         out = diffuse_evaluator.compute_npred()
-        assert out.shape == (2, 4, 5)
-        assert_allclose(out.sum(), 1.106403e12, rtol=1e-5)
-        assert_allclose(out[0, 0, 0], 5.586252e08, rtol=1e-5)
+        assert out.data.shape == (2, 4, 5)
+        assert_allclose(out.data.sum(), 1.106403e12, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 5.586508e08, rtol=1e-5)
 
 
 class TestSkyModelMapEvaluator:
@@ -333,16 +369,24 @@ class TestSkyModelMapEvaluator:
     def test_compute_dnde(evaluator):
         out = evaluator.compute_dnde()
         assert out.shape == (3, 4, 5)
-        assert out.unit == "cm-2 s-1 TeV-1 deg-2"
-        assert_allclose(out.value.sum(), 1.177907e-11, rtol=1e-5)
-        assert_allclose(out.value[0, 0, 0], 5.082551e-13, rtol=1e-5)
+        assert out.unit == "cm-2 s-1 TeV-1 sr-1"
+        assert_allclose(
+            out.to_value("cm-2 s-1 TeV-1 deg-2").sum(),
+            1.1788166328203174e-11,
+            rtol=1e-5,
+        )
+        assert_allclose(
+            out.to_value("cm-2 s-1 TeV-1 deg-2")[0, 0, 0],
+            5.087056282039508e-13,
+            rtol=1e-5,
+        )
 
     @staticmethod
     def test_compute_flux(evaluator):
         out = evaluator.compute_flux().to_value("cm-2 s-1")
         assert out.shape == (3, 4, 5)
-        assert_allclose(out.sum(), 1.290431e-12, rtol=1e-5)
-        assert_allclose(out[0, 0, 0], 4.626436e-14, rtol=1e-5)
+        assert_allclose(out.sum(), 1.291427605881036e-12, rtol=1e-5)
+        assert_allclose(out[0, 0, 0], 4.630845e-14, rtol=1e-5)
 
     @staticmethod
     def test_apply_psf(evaluator):
@@ -350,8 +394,8 @@ class TestSkyModelMapEvaluator:
         npred = evaluator.apply_exposure(flux)
         out = evaluator.apply_psf(npred)
         assert out.data.shape == (3, 4, 5)
-        assert_allclose(out.data.sum(), 2.25133e-06, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 2.405235e-08, rtol=1e-5)
+        assert_allclose(out.data.sum(), 2.2530737e-06, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 2.407252e-08, rtol=1e-5)
 
     @staticmethod
     def test_apply_edisp(evaluator):
@@ -359,12 +403,12 @@ class TestSkyModelMapEvaluator:
         npred = evaluator.apply_exposure(flux)
         out = evaluator.apply_edisp(npred)
         assert out.data.shape == (2, 4, 5)
-        assert_allclose(out.data.sum(), 3.273326e-06, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 4.626436e-08, rtol=1e-5)
+        assert_allclose(out.data.sum(), 3.2758538225058153e-06, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 4.630845e-08, rtol=1e-5)
 
     @staticmethod
     def test_compute_npred(evaluator):
         out = evaluator.compute_npred()
-        assert out.shape == (2, 4, 5)
-        assert_allclose(out.sum(), 6.25133e-06, rtol=1e-5)
-        assert_allclose(out[0, 0, 0], 1.240524e-07, rtol=1e-5)
+        assert out.data.shape == (2, 4, 5)
+        assert_allclose(out.data.sum(), 2.253073467739508e-06, rtol=1e-5)
+        assert_allclose(out.data[0, 0, 0], 2.407252e-08, rtol=1e-5)
