@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 from collections import namedtuple
 import numpy as np
@@ -19,7 +18,7 @@ __all__ = ["EventListBase", "EventList", "EventListLAT"]
 log = logging.getLogger(__name__)
 
 
-class EventListBase(object):
+class EventListBase:
     """Event list.
 
     This class represents the base for two different event lists:
@@ -73,7 +72,7 @@ class EventListBase(object):
 
         Parameters
         ----------
-        filename : `~gammapy.extern.pathlib.Path`, str
+        filename : `pathlib.Path`, str
             Filename
         """
         filename = make_path(filename)
@@ -556,6 +555,48 @@ class EventListBase(object):
         checker = EventListChecker(self)
         return checker.run(checks=checks)
 
+    def _get_coord_from_geom(self, geom):
+        """Return MapCoord of the EventList relative to an input geometry.
+
+        Parameters
+        ----------
+        geom : `~gammapy.maps.MapGeom`
+            the geom used to define the MapCoord
+
+        Returns
+        -------
+        mapcoord : `~gammapy.Map.MapCoord`
+        """
+        coord = dict(skycoord=self.radec)
+
+        cols = {k.upper(): v for k, v in self.table.columns.items()}
+
+        for axis in geom.axes:
+            try:
+                col = cols[axis.name.upper()]
+                coord[axis.name] = Quantity(col).to(axis.unit)
+            except KeyError:
+                raise KeyError("Column not found in event list: {!r}".format(axis.name))
+
+        return coord
+
+    def select_map_mask(self, mask):
+        """Return EventList contained in a Map mask.
+
+        Parameters
+        ----------
+        mask : `~gammapy.maps.Map`
+            the mask to be used
+
+        Returns
+        -------
+        eventlist : `~gammapy.data.EventList`
+        """
+        coord = self._get_coord_from_geom(mask.geom)
+        values = mask.get_by_coord(coord)
+        valid = values > 0
+        return self.select_row_subset(valid)
+
 
 class EventList(EventListBase):
     """Event list for IACT dataset
