@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 import numpy as np
 import logging
@@ -8,15 +7,13 @@ from astropy.table import Table
 from astropy.utils import lazyproperty
 from ..utils.scripts import make_path
 
-__all__ = [
-    'HDULocation',
-    'HDUIndexTable',
-]
+__all__ = ["HDULocation", "HDUIndexTable"]
 
 log = logging.getLogger(__name__)
 
-class HDULocation(object):
-    """HDU localisation and loading.
+
+class HDULocation:
+    """HDU localisation, loading and Gammapy object mapper.
 
     This represents one row in `HDUIndexTable`.
 
@@ -26,7 +23,9 @@ class HDULocation(object):
     See also :ref:`gadf:hdu-index`.
     """
 
-    def __init__(self, obs_id, hdu_type, hdu_class, base_dir, file_dir, file_name, hdu_name):
+    def __init__(
+        self, obs_id, hdu_type, hdu_class, base_dir, file_dir, file_name, hdu_name
+    ):
         self.obs_id = obs_id
         self.hdu_type = hdu_type
         self.hdu_class = hdu_class
@@ -40,30 +39,30 @@ class HDULocation(object):
         if not file:
             file = sys.stdout
 
-        print('OBS_ID = {}'.format(self.obs_id), file=file)
-        print('HDU_TYPE = {}'.format(self.hdu_type), file=file)
-        print('HDU_CLASS = {}'.format(self.hdu_class), file=file)
-        print('BASEDIR = {}'.format(self.base_dir), file=file)
-        print('FILE_DIR = {}'.format(self.file_dir), file=file)
-        print('FILE_NAME = {}'.format(self.file_name), file=file)
-        print('HDU_NAME = {}'.format(self.hdu_name), file=file)
+        print("OBS_ID = {}".format(self.obs_id), file=file)
+        print("HDU_TYPE = {}".format(self.hdu_type), file=file)
+        print("HDU_CLASS = {}".format(self.hdu_class), file=file)
+        print("BASE_DIR = {}".format(self.base_dir), file=file)
+        print("FILE_DIR = {}".format(self.file_dir), file=file)
+        print("FILE_NAME = {}".format(self.file_name), file=file)
+        print("HDU_NAME = {}".format(self.hdu_name), file=file)
 
     def path(self, abs_path=True):
-        """Full filename path."""
+        """Full filename path.
+
+        Include ``base_dir`` if ``abs_path`` is True.
+        """
         if abs_path:
-            path = make_path(self.base_dir) / self.file_dir / self.file_name
+            return make_path(self.base_dir) / self.file_dir / self.file_name
         else:
-            path = make_path(self.file_dir) / self.file_name
-
-        return path
-
-    def exists(self):
-        """Check if HDU exists."""
-        raise NotImplementedError
+            return make_path(self.file_dir) / self.file_name
 
     def get_hdu(self):
         """Get HDU."""
-        hdu_list = fits.open(str(self.path(abs_path=True)))
+        filename = str(self.path(abs_path=True))
+        # Here we're intentionally not calling `with fits.open`
+        # because we don't want the file to remain open.
+        hdu_list = fits.open(filename, memmap=False)
         return hdu_list[self.hdu_name]
 
     def load(self):
@@ -73,37 +72,46 @@ class HDULocation(object):
         """
         hdu_class = self.hdu_class
         filename = self.path()
-        hdu_name = self.hdu_name
+        hdu = self.hdu_name
 
-        if hdu_class == 'events':
+        if hdu_class == "events":
             from ..data import EventList
-            return EventList.read(filename, hdu=hdu_name)
-        elif hdu_class == 'gti':
+
+            return EventList.read(filename, hdu=hdu)
+        elif hdu_class == "gti":
             from ..data import GTI
-            return GTI.read(filename, hdu=hdu_name)
-        elif hdu_class == 'aeff_2d':
+
+            return GTI.read(filename, hdu=hdu)
+        elif hdu_class == "aeff_2d":
             from ..irf import EffectiveAreaTable2D
-            return EffectiveAreaTable2D.read(filename, hdu=hdu_name)
-        elif hdu_class == 'edisp_2d':
+
+            return EffectiveAreaTable2D.read(filename, hdu=hdu)
+        elif hdu_class == "edisp_2d":
             from ..irf import EnergyDispersion2D
-            return EnergyDispersion2D.read(filename, hdu=hdu_name)
-        elif hdu_class == 'psf_table':
+
+            return EnergyDispersion2D.read(filename, hdu=hdu)
+        elif hdu_class == "psf_table":
             from ..irf import PSF3D
-            return PSF3D.read(filename, hdu=hdu_name)
-        elif hdu_class == 'psf_3gauss':
+
+            return PSF3D.read(filename, hdu=hdu)
+        elif hdu_class == "psf_3gauss":
             from ..irf import EnergyDependentMultiGaussPSF
-            return EnergyDependentMultiGaussPSF.read(filename, hdu=hdu_name)
-        elif hdu_class == 'psf_king':
+
+            return EnergyDependentMultiGaussPSF.read(filename, hdu=hdu)
+        elif hdu_class == "psf_king":
             from ..irf import PSFKing
-            return PSFKing.read(filename, hdu=hdu_name)
-        elif hdu_class == 'bkg_2d':
-            from ..background import EnergyOffsetArray
-            return EnergyOffsetArray.read(filename, hdu=hdu_name, data_name='bkg')
-        elif hdu_class == 'bkg_3d':
+
+            return PSFKing.read(filename, hdu=hdu)
+        elif hdu_class == "bkg_2d":
+            from ..irf import Background2D
+
+            return Background2D.read(filename, hdu=hdu)
+        elif hdu_class == "bkg_3d":
             from ..irf import Background3D
-            return Background3D.read(filename, hdu=hdu_name)
+
+            return Background3D.read(filename, hdu=hdu)
         else:
-            raise ValueError('Invalid hdu_class: {}'.format(hdu_class))
+            raise ValueError("Invalid hdu_class: {}".format(hdu_class))
 
 
 class HDUIndexTable(Table):
@@ -111,41 +119,42 @@ class HDUIndexTable(Table):
 
     See :ref:`gadf:hdu-index`.
     """
-    VALID_HDU_TYPE = [
-        'events', 'gti', 'aeff', 'edisp', 'psf', 'bkg'
-    ]
+
+    VALID_HDU_TYPE = ["events", "gti", "aeff", "edisp", "psf", "bkg"]
     """Valid values for `HDU_TYPE`."""
 
     VALID_HDU_CLASS = [
-        'events', 'gti', 'aeff_2d', 'edisp_2d', 'psf_table',
-        'psf_3gauss', 'psf_king', 'bkg_2d', 'bkg_3d',
+        "events",
+        "gti",
+        "aeff_2d",
+        "edisp_2d",
+        "psf_table",
+        "psf_3gauss",
+        "psf_king",
+        "bkg_2d",
+        "bkg_3d",
     ]
     """Valid values for `HDU_CLASS`."""
 
-    # def __init__(self, base_dir, *args, **kwargs):
-    #     super(HDUIndexTable, self).__init(*args, **kwargs)
-    #     self.base_dir = base_dir
-
     @classmethod
     def read(cls, filename, **kwargs):
         """Read :ref:`gadf:hdu-index`.
 
         Parameters
         ----------
-        filename : `~gammapy.extern.pathlib.Path`, str
+        filename : `pathlib.Path`, str
             Filename
         """
         filename = make_path(filename)
-        table = super(HDUIndexTable, cls).read(str(filename), **kwargs)
-        # TODO: what is the best way to set BASE_DIR?
-        table.meta['BASE_DIR'] = filename.parent
+        table = super().read(str(filename), **kwargs)
+        table.meta["BASE_DIR"] = filename.parent.as_posix()
 
         return table
 
     @property
     def base_dir(self):
         """Base directory."""
-        return make_path(self.meta['BASE_DIR'])
+        return make_path(self.meta.get("BASE_DIR", ""))
 
     def hdu_location(self, obs_id, hdu_type=None, hdu_class=None):
         """Create `HDULocation` for a given selection.
@@ -171,14 +180,18 @@ class HDUIndexTable(Table):
         if len(idx) == 1:
             idx = idx[0]
         elif len(idx) == 0:
-            raise IndexError('No HDU found matching: OBS_ID = {}, HDU_TYPE = {}, HDU_CLASS = {}'
-                             ''.format(obs_id, hdu_type, hdu_class))
+            raise IndexError(
+                "No HDU found matching: OBS_ID = {}, HDU_TYPE = {}, HDU_CLASS = {}"
+                "".format(obs_id, hdu_type, hdu_class)
+            )
         else:
             idx = idx[0]
-            log.warn('Found multiple HDU matching: OBS_ID = {}, HDU_TYPE = {}, HDU_CLASS = {}.'
-                     ''.format(obs_id, hdu_type, hdu_class) +
-                     ' Returning the first entry, which has HDU_TYPE = {} and HDU_CLASS = {}'
-                     ''.format(self[idx]['HDU_TYPE'], self[idx]['HDU_CLASS']))
+            log.warning(
+                "Found multiple HDU matching: OBS_ID = {}, HDU_TYPE = {}, HDU_CLASS = {}."
+                "".format(obs_id, hdu_type, hdu_class)
+                + " Returning the first entry, which has HDU_TYPE = {} and HDU_CLASS = {}"
+                "".format(self[idx]["HDU_TYPE"], self[idx]["HDU_CLASS"])
+            )
 
         return self.location_info(idx)
 
@@ -188,25 +201,22 @@ class HDUIndexTable(Table):
         The goal is to give helpful error messages to the user.
         """
         if hdu_type is None and hdu_class is None:
-            raise ValueError('You have to specify `hdu_type` or `hdu_class`.')
-
-        # TODO: warn if both hdu_type and hdu_class are passed,
-        # stating that the more specific hdu_class will take precedence.
+            raise ValueError("You have to specify `hdu_type` or `hdu_class`.")
 
         if hdu_type and hdu_type not in self.VALID_HDU_TYPE:
-            msg = 'Invalid hdu_type: {}. '.format(hdu_type)
+            msg = "Invalid hdu_type: {}. ".format(hdu_type)
             valid = [str(_) for _ in self.VALID_HDU_TYPE]
-            msg += 'Valid values are: {}'.format(valid)
+            msg += "Valid values are: {}".format(valid)
             raise ValueError(msg)
 
         if hdu_class and hdu_class not in self.VALID_HDU_CLASS:
-            msg = 'Invalid hdu_class: {}. '.format(hdu_class)
+            msg = "Invalid hdu_class: {}. ".format(hdu_class)
             valid = [str(_) for _ in self.VALID_HDU_CLASS]
-            msg += 'Valid values are: {}'.format(valid)
+            msg += "Valid values are: {}".format(valid)
             raise ValueError(msg)
 
-        if obs_id not in self['OBS_ID']:
-            raise IndexError('No entry available with OBS_ID = {}'.format(obs_id))
+        if obs_id not in self["OBS_ID"]:
+            raise IndexError("No entry available with OBS_ID = {}".format(obs_id))
 
     def row_idx(self, obs_id, hdu_type=None, hdu_class=None):
         """Table row indices for a given selection.
@@ -225,7 +235,7 @@ class HDUIndexTable(Table):
         idx : list of int
             List of row indices matching the selection.
         """
-        selection = self['OBS_ID'] == obs_id
+        selection = self["OBS_ID"] == obs_id
 
         if hdu_class:
             is_hdu_class = self._hdu_class_stripped == hdu_class
@@ -241,48 +251,49 @@ class HDUIndexTable(Table):
     def location_info(self, idx):
         """Create `HDULocation` for a given row index."""
         row = self[idx]
-        location = HDULocation(
-            obs_id=row['OBS_ID'],
-            hdu_type=row['HDU_TYPE'].strip(),
-            hdu_class=row['HDU_CLASS'].strip(),
-            base_dir=self.base_dir,
-            file_dir=row['FILE_DIR'].strip(),
-            file_name=row['FILE_NAME'].strip(),
-            hdu_name=row['HDU_NAME'].strip(),
+        return HDULocation(
+            obs_id=row["OBS_ID"],
+            hdu_type=row["HDU_TYPE"].strip(),
+            hdu_class=row["HDU_CLASS"].strip(),
+            base_dir=self.base_dir.as_posix(),
+            file_dir=row["FILE_DIR"].strip(),
+            file_name=row["FILE_NAME"].strip(),
+            hdu_name=row["HDU_NAME"].strip(),
         )
-        return location
 
     @lazyproperty
     def _hdu_class_stripped(self):
-        return np.array([_.strip() for _ in self['HDU_CLASS']])
+        return np.array([_.strip() for _ in self["HDU_CLASS"]])
 
     @lazyproperty
     def _hdu_type_stripped(self):
-        return np.array([_.strip() for _ in self['HDU_TYPE']])
+        return np.array([_.strip() for _ in self["HDU_TYPE"]])
 
     @lazyproperty
     def obs_id_unique(self):
         """Observation IDs (unique)."""
-        return np.unique(np.sort(self['OBS_ID']))
+        return np.unique(np.sort(self["OBS_ID"]))
 
     @lazyproperty
     def hdu_type_unique(self):
         """HDU types (unique)."""
-        return list(np.unique(np.sort([_.strip() for _ in self['HDU_TYPE']])))
+        return list(np.unique(np.sort([_.strip() for _ in self["HDU_TYPE"]])))
 
     @lazyproperty
     def hdu_class_unique(self):
         """HDU classes (unique)."""
-        return list(np.unique(np.sort([_.strip() for _ in self['HDU_CLASS']])))
+        return list(np.unique(np.sort([_.strip() for _ in self["HDU_CLASS"]])))
 
-    def summary(self, file=None):
-        """Print some summary info to stdout."""
-        if not file:
-            file = sys.stdout
-
-        print('HDU index table:', file=file)
-        print('BASE_DIR: {}'.format(self.base_dir), file=file)
-        print('Rows: {}'.format(len(self)), file=file)
-        print('OBS_ID: {} -- {}'.format(self.obs_id_unique[0], self.obs_id_unique[-1]), file=file)
-        print('HDU_TYPE: {}'.format(self.hdu_type_unique), file=file)
-        print('HDU_CLASS: {}'.format(self.hdu_class_unique), file=file)
+    def summary(self):
+        """Summary report (str)"""
+        obs_id = self.obs_id_unique
+        return "\n".join(
+            [
+                "HDU index table:",
+                "BASE_DIR: {}".format(self.base_dir),
+                "Rows: {}".format(len(self)),
+                "OBS_ID: {} -- {}".format(obs_id[0], obs_id[-1]),
+                "HDU_TYPE: {}".format(self.hdu_type_unique),
+                "HDU_CLASS: {}".format(self.hdu_class_unique),
+            ]
+        )
