@@ -1,7 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
+from pathlib import Path
 import pytest
-from ...extern.pathlib import Path
 from ...utils.scripts import make_path
 from ...utils.testing import requires_data
 from ..hdu_index_table import HDUIndexTable
@@ -46,7 +45,7 @@ def test_hdu_index_table(hdu_index_table):
     assert hdu_index_table.summary().startswith("HDU index table")
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_hdu_index_table_hd_hap():
     """Test HESS HAP-HD data access."""
     hdu_index = HDUIndexTable.read("$GAMMAPY_DATA/hess-dl3-dr1/hdu-index.fits.gz")
