@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from astropy.units import Quantity
 from astropy.coordinates import Angle, SkyCoord, AltAz
@@ -428,7 +427,7 @@ def test_create_gti():
     )
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_observation_table_checker():
     path = "$GAMMAPY_DATA/cta-1dc/index/gps/obs-index.fits.gz"
     obs_table = ObservationTable.read(path)
