@@ -1,6 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Functions to compute TS images."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 import contextlib
 import warnings
@@ -9,11 +8,10 @@ from multiprocessing import Pool
 import numpy as np
 from scipy.optimize import newton, brentq
 from astropy.convolution import CustomKernel, Kernel2D
+from ..stats import cash, cash_sum_cython
 from ..utils.array import shape_2N, symmetric_crop_pad_width
 from ._test_statistics_cython import (
-    _cash_cython,
     _amplitude_bounds_cython,
-    _cash_sum_cython,
     _f_cash_root_cython,
     _x_best_leastsq,
 )
@@ -66,10 +64,12 @@ def f_cash(x, counts, background, model):
     model : `~numpy.ndarray`
         Source template (multiplied with exposure).
     """
-    return _cash_sum_cython(counts, background + x * FLUX_FACTOR * model)
+    return cash_sum_cython(
+        counts.ravel(), (background + x * FLUX_FACTOR * model).ravel()
+    )
 
 
-class TSMapEstimator(object):
+class TSMapEstimator:
     """
     Compute TS map using different optimization methods.
 
@@ -118,8 +118,8 @@ class TSMapEstimator(object):
 
         TS = \\left \\{
                  \\begin{array}{ll}
-                   -TS & : \\textnormal{if} \\ F < 0 \\\\
-                   \\ \\ TS & : \\textnormal{else}
+                   -TS & : \\text{if} \\ F < 0 \\\\
+                   \\ \\ TS & : \\text{else}
                  \\end{array}
                \\right.
 
@@ -166,13 +166,13 @@ class TSMapEstimator(object):
         Parameters
         ----------
         maps : dict
-            Input sky maps. Requires `counts`, `background` and `exposure` maps.
+            Input sky maps. Requires "counts", "background" and "exposure" maps.
         kernel : `astropy.convolution.Kernel2D`
             Source model kernel.
 
         Returns
         -------
-        flux_approx : `WcsNDMap`
+        flux_approx : `gammapy.maps.WcsNDMap`
             Approximate flux map.
         """
         flux = (maps["counts"].data - maps["background"].data) / maps["exposure"].data
@@ -186,13 +186,13 @@ class TSMapEstimator(object):
         Parameters
         ----------
         maps : dict
-            Input sky maps. Requires `background` and `exposure`.
+            Input sky maps. Requires "background" and "exposure".
         kernel : `astropy.convolution.Kernel2D`
             Source model kernel.
 
         Returns
         -------
-        mask : `WcsNDMap`
+        mask : `gammapy.maps.WcsNDMap`
             Mask map.
         """
         mask = np.zeros(maps["exposure"].data.shape, dtype=int)
@@ -209,6 +209,7 @@ class TSMapEstimator(object):
         # background, which doesn't make sense and causes the TS computation
         # to fail, this is a temporary fix
         mask[maps["background"].data == 0] = 0
+
         return maps["exposure"].copy(data=mask.astype("int"))
 
     @staticmethod
@@ -220,19 +221,19 @@ class TSMapEstimator(object):
         .. math::
             \sqrt{TS} = \left \{
             \begin{array}{ll}
-              -\sqrt{-TS} & : \textnormal{if} \ TS < 0 \\
-              \sqrt{TS} & : \textnormal{else}
+              -\sqrt{-TS} & : \text{if} \ TS < 0 \\
+              \sqrt{TS} & : \text{else}
             \end{array}
             \right.
 
         Parameters
         ----------
-        map_ts : `WcsNDMap`
+        map_ts : `gammapy.maps.WcsNDMap`
             Input TS map.
 
         Returns
         -------
-        sqrt_ts : `WcsNDMap`
+        sqrt_ts : `gammapy.maps.WcsNDMap`
             Sqrt(TS) map.
         """
         with np.errstate(invalid="ignore", divide="ignore"):
@@ -244,7 +245,7 @@ class TSMapEstimator(object):
         """
         Run TS map estimation.
 
-        Requires `counts`, `exposure` and `background` map to run.
+        Requires "counts", "exposure" and "background" map to run.
 
         Parameters
         ----------
@@ -273,15 +274,17 @@ class TSMapEstimator(object):
             )
 
         if downsampling_factor:
+            maps_downsampled = {}
+
             shape = maps["counts"].data.shape
             pad_width = symmetric_crop_pad_width(shape, shape_2N(shape))[0]
 
-            for name in maps:
-                maps[name] = maps[name].pad(pad_width)
+            for name, map_ in maps.items():
                 preserve_counts = name in ["counts", "background", "exclusion"]
-                maps[name] = maps[name].downsample(
+                maps_downsampled[name] = map_.pad(pad_width).downsample(
                     downsampling_factor, preserve_counts=preserve_counts
                 )
+            maps = maps_downsampled
 
         if not isinstance(kernel, Kernel2D):
             kernel = CustomKernel(kernel)
@@ -310,7 +313,7 @@ class TSMapEstimator(object):
         exposure = maps["exposure"].data.astype(float)
 
         # Compute null statistics per pixel for the whole image
-        c_0 = _cash_cython(counts, background)
+        c_0 = cash(counts, background)
 
         error_method = p["error_method"] if "flux_err" in which else "none"
         ul_method = p["ul_method"] if "flux_ul" in which else "none"
