@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal
 from ...utils.testing import requires_data
@@ -7,7 +6,7 @@ from ...detect import CWT, CWTKernels, CWTData
 from ...maps import Map
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestCWT:
     """Test CWT algorithm."""
 
@@ -126,7 +125,7 @@ class TestCWT:
         assert_allclose(transform_2d.sum(), 9.91731463861)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestCWTKernels:
     """Test CWTKernels"""
 
@@ -168,7 +167,7 @@ class TestCWTKernels:
         assert_equal(len(t), 13)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestCWTData:
     """
     Test CWTData class.
