@@ -1,20 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
-from scipy.interpolate import RegularGridInterpolator
 from astropy.table import Table
 from astropy.io import fits
-from astropy.units import Quantity
+from astropy import units as u
 from astropy.coordinates import Angle
+from astropy.utils import lazyproperty
 from ..utils.array import array_stats_str
 from ..utils.energy import Energy
 from ..utils.scripts import make_path
+from ..utils.interpolation import ScaledRegularGridInterpolator
 from .psf_table import TablePSF, EnergyDependentTablePSF
 
 __all__ = ["PSF3D"]
 
 
-class PSF3D(object):
+class PSF3D:
     """PSF with axes: energy, offset, rad.
 
     Data format specification: :ref:`gadf:psf_table`
@@ -47,8 +47,9 @@ class PSF3D(object):
         rad_lo,
         rad_hi,
         psf_value,
-        energy_thresh_lo=Quantity(0.1, "TeV"),
-        energy_thresh_hi=Quantity(100, "TeV"),
+        energy_thresh_lo=u.Quantity(0.1, "TeV"),
+        energy_thresh_hi=u.Quantity(100, "TeV"),
+        interp_kwargs=None,
     ):
         self.energy_lo = energy_lo.to("TeV")
         self.energy_hi = energy_hi.to("TeV")
@@ -59,6 +60,18 @@ class PSF3D(object):
         self.energy_thresh_lo = energy_thresh_lo.to("TeV")
         self.energy_thresh_hi = energy_thresh_hi.to("TeV")
 
+        self._interp_kwargs = interp_kwargs or {}
+
+    @lazyproperty
+    def _interpolate(self):
+        energy = self._energy_logcenter()
+        offset = self.offset.to("deg")
+        rad = self._rad_center()
+
+        return ScaledRegularGridInterpolator(
+            points=(rad, offset, energy), values=self.psf_value, **self._interp_kwargs
+        )
+
     def info(self):
         """Print some basic info.
         """
@@ -129,8 +142,8 @@ class PSF3D(object):
 
         opts = {}
         try:
-            opts["energy_thresh_lo"] = Quantity(table.meta["LO_THRES"], "TeV")
-            opts["energy_thresh_hi"] = Quantity(table.meta["HI_THRES"], "TeV")
+            opts["energy_thresh_lo"] = u.Quantity(table.meta["LO_THRES"], "TeV")
+            opts["energy_thresh_hi"] = u.Quantity(table.meta["HI_THRES"], "TeV")
         except KeyError:
             pass
 
@@ -184,7 +197,7 @@ class PSF3D(object):
         """
         self.to_fits().writeto(filename, *args, **kwargs)
 
-    def evaluate(self, energy=None, offset=None, rad=None, interp_kwargs=None):
+    def evaluate(self, energy=None, offset=None, rad=None):
         """Interpolate PSF value at a given offset and energy.
 
         Parameters
@@ -195,17 +208,12 @@ class PSF3D(object):
             Offset in the field of view
         rad : `~astropy.coordinates.Angle`
             Offset wrt source position
-        interp_kwargs : dict
-            option for interpolation for `~scipy.interpolate.RegularGridInterpolator`
 
         Returns
         -------
         values : `~astropy.units.Quantity`
             Interpolated value
         """
-        if not interp_kwargs:
-            interp_kwargs = dict(bounds_error=False, fill_value=None)
-
         if energy is None:
             energy = self._energy_logcenter()
         if offset is None:
@@ -213,21 +221,16 @@ class PSF3D(object):
         if rad is None:
             rad = self._rad_center()
 
-        energy = Energy(energy).to("TeV")
-        offset = Angle(offset).to("deg")
-        rad = Angle(rad).to("deg")
-
-        energy_bin = self._energy_logcenter()
-
-        offset_bin = self.offset.to("deg")
-        rad_bin = self._rad_center()
-        points = (rad_bin, offset_bin, energy_bin)
-        interpolator = RegularGridInterpolator(points, self.psf_value, **interp_kwargs)
-        rr, off, ee = np.meshgrid(rad.value, offset.value, energy.value, indexing="ij")
-        shape = ee.shape
-        pix_coords = np.column_stack([rr.flat, off.flat, ee.flat])
-        data_interp = interpolator(pix_coords)
-        return Quantity(data_interp.reshape(shape), self.psf_value.unit)
+        rad = np.atleast_1d(u.Quantity(rad))
+        offset = np.atleast_1d(u.Quantity(offset))
+        energy = np.atleast_1d(u.Quantity(energy))
+        return self._interpolate(
+            (
+                rad[:, np.newaxis, np.newaxis],
+                offset[np.newaxis, :, np.newaxis],
+                energy[np.newaxis, np.newaxis, :],
+            )
+        )
 
     def to_energy_dependent_table_psf(self, theta="0 deg", rad=None, exposure=None):
         """
@@ -257,14 +260,12 @@ class PSF3D(object):
         else:
             rad = Angle(rad)
 
-        psf_value = self.evaluate(offset=theta, rad=rad)
-        psf_value = psf_value[:, 0, :].transpose()
-
+        psf_value = self.evaluate(offset=theta, rad=rad).squeeze()
         return EnergyDependentTablePSF(
-            energy=energies, rad=rad, exposure=exposure, psf_value=psf_value
+            energy=energies, rad=rad, exposure=exposure, psf_value=psf_value.T
         )
 
-    def to_table_psf(self, energy, theta="0 deg", interp_kwargs=None, **kwargs):
+    def to_table_psf(self, energy, theta="0 deg", **kwargs):
         """Create `~gammapy.irf.TablePSF` at one given energy.
 
         Parameters
@@ -273,17 +274,15 @@ class PSF3D(object):
             Energy
         theta : `~astropy.coordinates.Angle`
             Offset in the field of view. Default theta = 0 deg
-        interp_kwargs : dict
-            Option for interpolation for `~scipy.interpolate.RegularGridInterpolator`
 
         Returns
         -------
         psf : `~gammapy.irf.TablePSF`
             Table PSF
         """
-        energy = Quantity(energy)
+        energy = u.Quantity(energy)
         theta = Angle(theta)
-        psf_value = self.evaluate(energy, theta, interp_kwargs=interp_kwargs).squeeze()
+        psf_value = self.evaluate(energy, theta).squeeze()
         rad = self._rad_center()
         return TablePSF(rad, psf_value, **kwargs)
 
@@ -306,31 +305,15 @@ class PSF3D(object):
         radius : `~astropy.units.Quantity`
             Containment radius in deg
         """
-        energy = Quantity(energy)
-        if energy.ndim == 0:
-            energy = Quantity([energy.value], energy.unit)
+        energy = np.atleast_1d(u.Quantity(energy))
+        theta = np.atleast_1d(u.Quantity(theta))
 
-        theta = Angle(theta)
-        if theta.ndim == 0:
-            theta = Quantity([theta.value], theta.unit)
+        radii = []
+        for t in theta:
+            psf = self.to_energy_dependent_table_psf(theta=t)
+            radii.append(psf.containment_radius(energy, fraction=fraction))
 
-        unit = None
-        radius = np.zeros((energy.size, theta.size))
-        for e in range(energy.size):
-            for t in range(theta.size):
-                try:
-                    psf = self.to_table_psf(energy[e], theta[t], interp_kwargs)
-                except:
-                    # This can raise an `error` from scipy UnivariateSpline:
-                    # error: (xb<=x[0]) failed for 2nd keyword xb: fpcurf0:xb=nan
-                    # Not sure what type exactly or how to catch it.
-                    radius[e, t] = np.nan
-                    continue
-                r = psf.containment_radius(fraction)
-                radius[e, t] = r.value
-                unit = r.unit
-
-        return Quantity(radius.squeeze(), unit)
+        return u.Quantity(radii).T.squeeze()
 
     def plot_containment_vs_energy(
         self, fractions=[0.68, 0.95], thetas=Angle([0, 1], "deg"), ax=None
@@ -345,8 +328,8 @@ class PSF3D(object):
 
         for theta in thetas:
             for fraction in fractions:
-                radius = self.containment_radius(energy, theta, fraction).squeeze()
-                label = "{} deg, {:.1f}%".format(theta, 100 * fraction)
+                radius = self.containment_radius(energy, theta, fraction)
+                label = "{} deg, {:.1f}%".format(theta.deg, 100 * fraction)
                 ax.plot(energy.value, radius.value, label=label)
 
         ax.semilogx()
@@ -354,7 +337,7 @@ class PSF3D(object):
         ax.set_xlabel("Energy (TeV)")
         ax.set_ylabel("Containment radius (deg)")
 
-    def plot_psf_vs_rad(self, theta="0 deg", energy=Quantity(1, "TeV")):
+    def plot_psf_vs_rad(self, theta="0 deg", energy=u.Quantity(1, "TeV")):
         """Plot PSF vs rad.
 
         Parameters
@@ -412,7 +395,7 @@ class PSF3D(object):
             self._plot_safe_energy_range(ax)
 
         if add_cbar:
-            label = "Containment radius R{0:.0f} ({1})" "".format(
+            label = "Containment radius R{:.0f} ({})" "".format(
                 100 * fraction, containment.unit
             )
             ax.figure.colorbar(caxes, ax=ax, label=label)
@@ -425,7 +408,7 @@ class PSF3D(object):
         omin = self.offset.value.min()
         omax = self.offset.value.max()
         ax.hlines(y=esafe.value, xmin=omin, xmax=omax)
-        label = "Safe energy threshold: {0:3.2f}".format(esafe)
+        label = "Safe energy threshold: {:3.2f}".format(esafe)
         ax.text(x=0.1, y=0.9 * esafe.value, s=label, va="top")
 
     def peek(self, figsize=(15, 5)):
