@@ -1,12 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 from numpy.testing import assert_allclose
 from astropy.units import Quantity
 from ...utils.testing import requires_data
 from ..io import load_cta_irfs
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_cta_irf():
     """Test that CTA IRFs can be loaded and evaluated."""
     irf = load_cta_irfs(
