@@ -1,14 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal
 from astropy.coordinates import Angle, SkyCoord
-from ..irf_reduce import make_psf, make_mean_psf, make_mean_edisp
+import astropy.units as u
+from ..irf_reduce import (
+    make_psf,
+    make_mean_psf,
+    make_mean_edisp,
+    apply_containment_fraction,
+    compute_energy_thresholds,
+)
+from ..effective_area import EffectiveAreaTable
+from ..energy_dispersion import EnergyDispersion
+from ..psf_table import EnergyDependentTablePSF, TablePSF
 from ...data import DataStore, Observations
 from ...utils.testing import requires_data, assert_quantity_allclose
 from ...utils.energy import Energy, EnergyBounds
-from ...spectrum.models import PowerLaw
 
 
 @pytest.fixture(scope="session")
@@ -16,7 +24,7 @@ def data_store():
     return DataStore.from_dir("$GAMMAPY_DATA/hess-dl3-dr1/")
 
 
-@requires_data("gammapy-data")
+@requires_data()
 @pytest.mark.parametrize(
     "pars",
     [
@@ -91,32 +99,18 @@ def test_make_psf(pars, data_store):
     assert_allclose(psf.psf_value.value[15, 50], pars["psf_value"], rtol=1e-3)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_make_mean_psf(data_store):
     position = SkyCoord(83.63, 22.01, unit="deg")
-    obs1 = data_store.obs(23523)
-    obs2 = data_store.obs(23526)
-    energy = EnergyBounds.equal_log_spacing(1, 10, 100, "TeV")
-    energy_band = Energy([energy[0].value, energy[-1].value], energy.unit)
 
-    psf1 = make_psf(obs1, position=position, energy=energy, rad=None)
-    psf2 = make_psf(obs2, position=position, energy=energy, rad=None)
+    observations = data_store.get_observations([23523, 23526])
+    psf = make_mean_psf(observations, position=position)
 
-    spectrum = PowerLaw(index=2.3)
-    psf1_int = psf1.table_psf_in_energy_band(energy_band, spectrum=spectrum)
-    psf2_int = psf2.table_psf_in_energy_band(energy_band, spectrum=spectrum)
-    observations = Observations([obs1, obs2])
-    psf_tot = make_mean_psf(observations, position=position, energy=energy)
-    psf_tot_int = psf_tot.table_psf_in_energy_band(energy_band, spectrum=spectrum)
-
-    # Check that the mean PSF is consistent with the individual PSFs
-    # (in this case the R68 of the mean PSF is in between the R68 of the individual PSFs)
-    assert_allclose(psf1_int.containment_radius(0.68).deg, 0.122769, rtol=1e-3)
-    assert_allclose(psf2_int.containment_radius(0.68).deg, 0.111322, rtol=1e-3)
-    assert_allclose(psf_tot_int.containment_radius(0.68).deg, 0.117134, rtol=1e-3)
+    assert not np.isnan(psf.psf_value.value).any()
+    assert_allclose(psf.psf_value.value[22, 22], 12206.1665)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_make_mean_edisp(data_store):
     position = SkyCoord(83.63, 22.01, unit="deg")
 
@@ -128,8 +122,8 @@ def test_make_mean_edisp(data_store):
     e_reco = EnergyBounds.equal_log_spacing(0.5, 100, 15, "TeV")
     rmf = make_mean_edisp(observations, position=position, e_true=e_true, e_reco=e_reco)
 
-    assert len(rmf.e_true.nodes) == 80
-    assert len(rmf.e_reco.nodes) == 15
+    assert len(rmf.e_true.center) == 80
+    assert len(rmf.e_reco.center) == 15
     assert_quantity_allclose(rmf.data.data[53, 8], 0.056, atol=2e-2)
 
     rmf2 = make_mean_edisp(
@@ -150,3 +144,72 @@ def test_make_mean_edisp(data_store):
     i = np.where(rmf.data.evaluate(e_reco=Energy(40, "TeV")) != 0)[0]
     i2 = np.where(rmf2.data.evaluate(e_reco=Energy(40, "TeV")) != 0)[0]
     assert_equal(i, i2)
+
+
+def test_apply_containment_fraction():
+    n_edges_energy = 5
+    energy = EnergyBounds.equal_log_spacing(0.1, 10.0, nbins=n_edges_energy, unit="TeV")
+    area = np.ones(n_edges_energy) * 4 * u.m ** 2
+    aeff = EffectiveAreaTable(energy.lower_bounds, energy.upper_bounds, data=area)
+
+    nrad = 100
+    rad = Angle(np.linspace(0, 0.5, nrad), "deg")
+    psf_table = TablePSF.from_shape(shape="disk", width="0.2 deg", rad=rad)
+    psf_values = (
+        np.resize(psf_table.psf_value.value, (n_edges_energy, nrad))
+        * psf_table.psf_value.unit
+    )
+    edep_psf_table = EnergyDependentTablePSF(
+        aeff.energy.center, rad, psf_value=psf_values
+    )
+
+    new_aeff = apply_containment_fraction(aeff, edep_psf_table, Angle("0.1 deg"))
+
+    assert_allclose(new_aeff.data.data.value, 1.0, rtol=5e-4)
+    assert new_aeff.data.data.unit == "m2"
+
+
+@requires_data("gammapy-data")
+def test_compute_thresholds_from_crab_data():
+    """Obs read from file"""
+    arffile = "$GAMMAPY_DATA/joint-crab/spectra/hess/arf_obs23523.fits"
+    rmffile = "$GAMMAPY_DATA/joint-crab/spectra/hess/rmf_obs23523.fits"
+
+    aeff = EffectiveAreaTable.read(arffile)
+    edisp = EnergyDispersion.read(rmffile)
+
+    thresh_lo, thresh_hi = compute_energy_thresholds(
+        aeff=aeff,
+        edisp=edisp,
+        method_lo="energy_bias",
+        method_hi="none",
+        bias_percent_lo=10,
+        bias_percent_hi=10,
+    )
+
+    assert_allclose(thresh_lo.to("TeV").value, 0.9174, rtol=1e-4)
+    assert_allclose(thresh_hi.to("TeV").value, 100.0, rtol=1e-4)
+
+
+def test_compute_thresholds_from_parametrization():
+    energy = np.logspace(-2, 2.0, 100) * u.TeV
+    aeff = EffectiveAreaTable.from_parametrization(energy=energy)
+    edisp = EnergyDispersion.from_gauss(e_true=energy, e_reco=energy, sigma=0.2, bias=0)
+
+    thresh_lo, thresh_hi = compute_energy_thresholds(
+        aeff=aeff,
+        edisp=edisp,
+        method_lo="area_max",
+        method_hi="area_max",
+        area_percent_lo=10,
+        area_percent_hi=90,
+    )
+
+    assert_allclose(thresh_lo.to("TeV").value, 0.18557, rtol=1e-4)
+    assert_allclose(thresh_hi.to("TeV").value, 43.818, rtol=1e-4)
+
+    thresh_lo, thresh_hi = compute_energy_thresholds(
+        aeff=aeff, edisp=edisp, method_hi="area_max", area_percent_hi=70
+    )
+
+    assert_allclose(thresh_hi.to("TeV").value, 100.0, rtol=1e-4)
