@@ -1,12 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 from numpy.testing import assert_allclose
 from ...utils.testing import requires_data
 from ..psf_3d import PSF3D
 from ..psf_check import PSF3DChecker
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestPSF3DChecker:
     def setup(self):
         filename = "$GAMMAPY_DATA/tests/hess-hap-hd-prod3/psf_table.fits.gz"
