@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
@@ -8,15 +7,14 @@ from astropy.coordinates import SkyCoord, Angle
 import astropy.units as u
 from ..wcs import WcsGeom, _check_width
 from ..geom import MapAxis
+from ..base import Map
 
-pytest.importorskip("scipy")
-
-axes1 = [MapAxis(np.logspace(0., 3., 3), interp="log", name="energy")]
+axes1 = [MapAxis(np.logspace(0.0, 3.0, 3), interp="log", name="energy")]
 axes2 = [
-    MapAxis(np.logspace(0., 3., 3), interp="log", name="energy"),
-    MapAxis(np.logspace(1., 3., 4), interp="lin"),
+    MapAxis(np.logspace(0.0, 3.0, 3), interp="log", name="energy"),
+    MapAxis(np.logspace(1.0, 3.0, 4), interp="lin"),
 ]
-skydir = SkyCoord(110., 75.0, unit="deg", frame="icrs")
+skydir = SkyCoord(110.0, 75.0, unit="deg", frame="icrs")
 
 wcs_allsky_test_geoms = [
     (None, 10.0, "GAL", "AIT", skydir, None),
@@ -82,7 +80,7 @@ def test_wcsgeom_test_coord_to_idx(npix, binsz, coordsys, proj, skydir, axes):
 
     if not geom.is_allsky:
         coords = geom.center_coord[:2] + tuple([ax.center[0] for ax in geom.axes])
-        coords[0][...] += 2.0 * np.max(geom.width[0])
+        coords[0][...] += 2.0 * np.max(geom.width[0].to_value("deg"))
         idx = geom.coord_to_idx(coords)
         assert_allclose(np.full_like(coords[0], -1, dtype=int), idx[0])
         idx = geom.coord_to_idx(coords, clip=True)
@@ -131,7 +129,8 @@ def test_wcsgeom_contains(npix, binsz, coordsys, proj, skydir, axes):
         npix=npix, binsz=binsz, skydir=skydir, proj=proj, coordsys=coordsys, axes=axes
     )
     coords = geom.get_coord()
-    coords = [c[np.isfinite(c)] for c in coords]
+    m = np.isfinite(coords[0])
+    coords = [c[m] for c in coords]
     assert_allclose(geom.contains(coords), np.ones(coords[0].shape, dtype=bool))
 
     if axes is not None:
@@ -169,6 +168,20 @@ def test_wcsgeom_solid_angle():
     assert_allclose(solid_angle.value[0, 9, 5], 0.0003038, rtol=1e-3)
 
 
+def test_wcsgeom_solid_angle_symmetry():
+    geom = WcsGeom.create(
+        skydir=(0, 0), coordsys="GAL", npix=(3, 3), binsz=20.0 * u.deg
+    )
+
+    sa = geom.solid_angle()
+
+    assert_allclose(sa[1, :], sa[1, 0])  # Constant along lon
+    assert_allclose(sa[0, 1], sa[2, 1])  # Symmetric along lat
+    with pytest.raises(AssertionError):
+        # Not constant along lat due to changes in solid angle (great circle)
+        assert_allclose(sa[:, 1], sa[0, 1])
+
+
 def test_wcsgeom_solid_angle_ait():
     # Pixels that don't correspond to locations on ths sky
     # should have solid angles set to NaN
@@ -202,6 +215,21 @@ def test_wcsgeom_separation():
     assert_allclose(separation.value[0, 0], 0.7106291438079875)
 
 
+def test_cutout():
+    geom = WcsGeom.create(
+        skydir=(0, 0),
+        npix=10,
+        binsz=0.1,
+        coordsys="GAL",
+        proj="CAR",
+        axes=[MapAxis.from_edges([0, 2, 3])],
+    )
+    position = SkyCoord(0.1, 0.2, unit="deg", frame="galactic")
+    cutout_geom = geom.cutout(position=position, width=2 * 0.3 * u.deg, mode="trim")
+    assert_allclose(cutout_geom.center_coord, (0.1, 0.2, 2.0))
+    assert cutout_geom.data_shape == (2, 6, 6)
+
+
 def test_wcsgeom_get_coord():
     geom = WcsGeom.create(
         skydir=(0, 0), npix=(4, 3), binsz=1, coordsys="GAL", proj="CAR"
@@ -259,6 +287,28 @@ def test_region_mask():
     assert np.sum(mask) == 8
 
 
+def test_energy_mask():
+    energy_axis = MapAxis.from_nodes(
+        [1, 10, 100], interp="log", name="energy", unit="TeV"
+    )
+    geom = WcsGeom.create(npix=(1, 1), binsz=1, proj="CAR", axes=[energy_axis])
+
+    mask = geom.energy_mask(emin=3 * u.TeV)
+    assert not mask[0, 0, 0]
+    assert mask[1, 0, 0]
+    assert mask[2, 0, 0]
+
+    mask = geom.energy_mask(emax=30 * u.TeV)
+    assert mask[0, 0, 0]
+    assert mask[1, 0, 0]
+    assert not mask[2, 0, 0]
+
+    mask = geom.energy_mask(emin=3 * u.TeV, emax=30 * u.TeV)
+    assert not mask[0, 0, 0]
+    assert not mask[-1, 0, 0]
+    assert mask[1, 0, 0]
+
+
 @pytest.mark.parametrize(
     ("width", "out"),
     [
@@ -281,10 +331,75 @@ def test_check_width(width, out):
     assert isinstance(width[1], float)
     assert width == out
 
-    geom = WcsGeom.create(width=width, binsz=1.)
+    geom = WcsGeom.create(width=width, binsz=1.0)
     assert tuple(geom.npix) == out
 
 
 def test_check_width_bad_input():
     with pytest.raises(IndexError):
         _check_width(width=(10,))
+
+
+def test_get_axis_index_by_name():
+    e_axis = MapAxis.from_edges([1, 5], name="energy")
+    geom = WcsGeom.create(width=5, binsz=1.0, axes=[e_axis])
+    assert geom.get_axis_index_by_name("Energy") == 0
+    with pytest.raises(ValueError):
+        geom.get_axis_index_by_name("time")
+
+
+test_axis1 = [MapAxis(nodes=(1, 2, 3, 4), unit="TeV", node_type="center")]
+test_axis2 = [
+    MapAxis(nodes=(1, 2, 3, 4), unit="TeV", node_type="center"),
+    MapAxis(nodes=(1, 2, 3), unit="TeV", node_type="center"),
+]
+
+skydir2 = SkyCoord(110.0, 75.0 + 1e-8, unit="deg", frame="icrs")
+skydir3 = SkyCoord(110.0, 75.0 + 1e-3, unit="deg", frame="icrs")
+
+compatibility_test_geoms = [
+    (10, 0.1, "GAL", "CAR", skydir, test_axis1, True),
+    (10, 0.1, "GAL", "CAR", skydir2, test_axis1, True),
+    (10, 0.1, "GAL", "CAR", skydir3, test_axis1, False),
+    (10, 0.1, "GAL", "TAN", skydir, test_axis1, False),
+    (8, 0.1, "GAL", "CAR", skydir, test_axis1, False),
+    (10, 0.1, "GAL", "CAR", skydir, test_axis2, False),
+    (10, 0.1, "GAL", "CAR", skydir.galactic, test_axis1, True),
+]
+
+
+@pytest.mark.parametrize(
+    ("npix", "binsz", "coordsys", "proj", "skypos", "axes", "result"),
+    compatibility_test_geoms,
+)
+def test_wcs_geom_equal(npix, binsz, coordsys, proj, skypos, axes, result):
+    geom0 = WcsGeom.create(
+        skydir=skydir, npix=10, binsz=0.1, proj="CAR", coordsys="GAL", axes=test_axis1
+    )
+    geom1 = WcsGeom.create(
+        skydir=skypos, npix=npix, binsz=binsz, proj=proj, coordsys=coordsys, axes=axes
+    )
+
+    assert (geom0 == geom1) is result
+    assert (geom0 != geom1) is not result
+
+
+@pytest.mark.parametrize("node_type", ["edges", "center"])
+@pytest.mark.parametrize("interp", ["log", "lin", "sqrt"])
+def test_read_write(tmpdir, node_type, interp):
+    # Regression test for MapAxis interp and node_type FITS serialisation
+    # https://github.com/gammapy/gammapy/issues/1887
+    e_ax = MapAxis([1, 2], interp, "energy", node_type, "TeV")
+    t_ax = MapAxis([3, 4], interp, "time", node_type, "s")
+    m = Map.create(binsz=1, npix=10, axes=[e_ax, t_ax], unit="m2")
+
+    # Check what Gammapy writes in the FITS header
+    header = m.make_hdu().header
+    assert header["INTERP1"] == interp
+    assert header["INTERP2"] == interp
+
+    # Check that all MapAxis properties are preserved on FITS I/O
+    filename = str(tmpdir / "geom_test.fits")
+    m.write(filename, overwrite=True)
+    m2 = Map.read(filename)
+    assert m2.geom == m.geom
