@@ -1,7 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Command line tool to perform actions on jupyter notebooks."""
-
-from __future__ import absolute_import, division, print_function, unicode_literals
 import click
 import logging
 import os
@@ -26,7 +24,6 @@ OFF = ["GAMMA_CAT", "GAMMAPY_DATA", "GAMMAPY_EXTRA", "GAMMAPY_FERMI_LAT_DATA"]
 @click.option("--kernel", default="python3", help="Kernel name", show_default=True)
 def cli_jupyter_run(ctx, tutor, kernel):
     """Execute Jupyter notebooks."""
-
     with environment(OFF, tutor, ctx):
         for path in ctx.obj["paths"]:
             notebook_test(path, kernel)
@@ -34,33 +31,39 @@ def cli_jupyter_run(ctx, tutor, kernel):
 
 def execute_notebook(path, kernel="python3", loglevel=30):
     """Execute a Jupyter notebook."""
-
-    if sys.version_info[0] < 3 and kernel == "python3":
-        kernel = "python2"
-
     try:
-        t = time.time()
-        cmd = [
-            sys.executable,
-            "-m",
-            "jupyter",
-            "nbconvert",
-            "--allow-errors",
-            "--log-level={}".format(loglevel),
-            "--ExecutePreprocessor.timeout=None",
-            "--ExecutePreprocessor.kernel_name={}".format(kernel),
-            "--to",
-            "notebook",
-            "--inplace",
-            "--execute",
-            "{}".format(path),
-        ]
-        subprocess.call(cmd)
-        t = (time.time() - t) / 60
-        log.info("   ... Executing duration: {:.2f} mn".format(t))
-    except Exception as ex:
+        import jupyter
+
+        jupyter_module = "jupyter"
+    except ImportError:
+        import jupyter_core
+
+        jupyter_module = "jupyter_core"
+
+    t = time.time()
+    cmd = [
+        sys.executable,
+        "-m",
+        jupyter_module,
+        "nbconvert",
+        "--allow-errors",
+        "--log-level={}".format(loglevel),
+        "--ExecutePreprocessor.timeout=None",
+        "--ExecutePreprocessor.kernel_name={}".format(kernel),
+        "--to",
+        "notebook",
+        "--inplace",
+        "--execute",
+        "{}".format(path),
+    ]
+
+    if subprocess.call(cmd):
         log.error("Error executing file {}".format(str(path)))
-        log.error(ex)
+        return False
+    else:
+        t = time.time() - t
+        log.info("   ... Executing duration: {:.1f} seconds".format(t))
+        return True
 
 
 @click.command(name="strip")
@@ -73,12 +76,13 @@ def cli_jupyter_strip(ctx):
         rawnb = nbformat.read(str(path), as_version=nbformat.NO_CONVERT)
 
         for cell in rawnb.cells:
+            cell["metadata"].pop("pycharm", None)
             if cell["cell_type"] == "code":
                 cell["execution_count"] = None
                 cell["outputs"] = []
 
         nbformat.write(rawnb, str(path))
-        log.info("Jupyter notebook {} stripped out.".format(str(path)))
+        log.info("Strip output cells in notebook: {}".format(str(path)))
 
 
 @click.command(name="black")
@@ -93,11 +97,16 @@ def cli_jupyter_black(ctx):
         blacknb.blackformat()
         rawnb = blacknb.rawnb
         nbformat.write(rawnb, str(path))
-        log.info("Jupyter notebook {} blacked.".format(str(path)))
+        log.info("Applied black to notebook: {}".format(str(path)))
 
 
 class BlackNotebook:
-    """Manage the process of black formatting."""
+    """Manage the process of black formatting.
+
+    Probably this will become available directly in the future.
+
+    See https://github.com/ambv/black/issues/298#issuecomment-476960082
+    """
 
     MAGIC_TAG = "###-MAGIC TAG-"
 
@@ -106,7 +115,7 @@ class BlackNotebook:
 
     def blackformat(self):
         """Format code cells."""
-        from black import format_str
+        import black
 
         for cell in self.rawnb.cells:
             fmt = cell["source"]
@@ -114,7 +123,9 @@ class BlackNotebook:
                 try:
                     fmt = "\n".join(self.tag_magics(fmt))
                     has_semicolon = fmt.endswith(";")
-                    fmt = format_str(src_contents=fmt, line_length=79).rstrip()
+                    fmt = black.format_str(
+                        src_contents=fmt, mode=black.FileMode(line_length=79)
+                    ).rstrip()
                     if has_semicolon:
                         fmt += ";"
                 except Exception as ex:
@@ -124,7 +135,6 @@ class BlackNotebook:
 
     def tag_magics(self, cellcode):
         """Comment magic commands."""
-
         lines = cellcode.splitlines(False)
         for line in lines:
             if line.startswith("%") or line.startswith("!"):
@@ -146,7 +156,6 @@ class BlackNotebook:
 @click.option("--kernel", default="python3", help="Kernel name", show_default=True)
 def cli_jupyter_test(ctx, tutor, kernel):
     """Check if Jupyter notebooks are broken."""
-
     with environment(OFF, tutor, ctx):
         for path in ctx.obj["paths"]:
             notebook_test(path, kernel)
@@ -156,10 +165,10 @@ def notebook_test(path, kernel="python3"):
     """Execute and parse a Jupyter notebook exposing broken cells."""
     import nbformat
 
-    passed = True
     log.info("   ... EXECUTING {}".format(str(path)))
-    execute_notebook(path, kernel)
+    passed = execute_notebook(path, kernel)
     rawnb = nbformat.read(str(path), as_version=nbformat.NO_CONVERT)
+    report = ""
 
     for cell in rawnb.cells:
         if "outputs" in cell.keys():
@@ -188,9 +197,7 @@ def notebook_test(path, kernel="python3"):
 
 
 class environment:
-    """
-    Helper for setting environmental variables
-    """
+    """Helper for setting environmental variables."""
 
     def __init__(self, envs, tutor, ctx):
         self.envs = envs
@@ -198,7 +205,6 @@ class environment:
         self.ctx = ctx
 
     def __enter__(self):
-
         self.old = os.environ
         if self.tutor:
             for item in self.envs:
