@@ -1,14 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """1D spectrum analysis."""
 from .core import *
-from .observation import *
 from .cosmic_ray import *
 from .crab import *
-from .energy_group import *
+from .dataset import *
 from .flux_point import *
 from .utils import *
 from .extract import *
 from .simulation import *
-from .fit import *
-from .results import *
 from .sensitivity import *
