@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 from collections import OrderedDict
 import numpy as np
@@ -8,13 +7,13 @@ from astropy import units as u
 from astropy.io.registry import IORegistryError
 from ..utils.scripts import make_path
 from ..utils.table import table_standardise_units_copy, table_from_row_data
-from ..utils.fitting import Fit
-from ..utils.interpolation import ScaledRegularGridInterpolator
+from ..utils.interpolation import interpolate_likelihood_profile
+from ..utils.fitting import Dataset, Datasets, Fit
 from .models import PowerLaw, ScaleModel
 from .powerlaw import power_law_integral_flux
-from .observation import SpectrumObservationList, SpectrumObservation
+from .dataset import SpectrumDatasetOnOff
 
-__all__ = ["FluxPoints", "FluxPointEstimator", "FluxPointFit"]
+__all__ = ["FluxPoints", "FluxPointsEstimator", "FluxPointsDataset"]
 
 log = logging.getLogger(__name__)
 
@@ -59,19 +58,7 @@ DEFAULT_UNIT = OrderedDict(
 )
 
 
-def _interp_likelihood_profile(norm_scan, dloglike_scan, norm):
-    """Helper function to interpolate likelihood profiles"""
-    # likelihood profiles are typically of parabolic shape, so we use a
-    # sqrt scaling of the values and perform linear interpolation on the scaled
-    # values
-    sign = np.sign(np.gradient(dloglike_scan))
-    interp = ScaledRegularGridInterpolator(
-        points=(norm_scan,), values=sign * dloglike_scan, values_scale="sqrt"
-    )
-    return interp((norm,))
-
-
-class FluxPoints(object):
+class FluxPoints:
     """Flux points container.
 
     The supported formats are described here: :ref:`gadf:flux-points`
@@ -260,7 +247,6 @@ class FluxPoints(object):
 
         Examples
         --------
-
         >>> from gammapy.spectrum import FluxPoints
         >>> filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points.fits'
         >>> flux_points = FluxPoints.read(filename)
@@ -465,7 +451,8 @@ class FluxPoints(object):
                 "Missing columns for sed type '{}':" " {}".format(sed_type, missing)
             )
 
-    def _get_y_energy_unit(self, y_unit):
+    @staticmethod
+    def _get_y_energy_unit(y_unit):
         """Get energy part of the given y unit."""
         try:
             return [_ for _ in y_unit.bases if _.physical_type == "energy"][0]
@@ -703,8 +690,8 @@ class FluxPoints(object):
         if y_values is None:
             ref_values = self.table["ref_" + self.sed_type].quantity
             y_values = np.logspace(
-                np.log10(ref_values.value.min()) - 1,
-                np.log10(ref_values.value.max()) + 1,
+                np.log10(0.2 * ref_values.value.min()),
+                np.log10(5 * ref_values.value.max()),
                 500,
             )
             y_values = u.Quantity(y_values, y_unit, copy=False)
@@ -714,14 +701,13 @@ class FluxPoints(object):
         # Compute likelihood "image" one energy bin at a time
         # by interpolating e2dnde at the log bin centers
         z = np.empty((len(self.table), len(y_values)))
-        for idx in range(len(self.table)):
+        for idx, row in enumerate(self.table):
             y_ref = self.table["ref_" + self.sed_type].quantity[idx]
             norm = (y_values / y_ref).to_value("")
-            norm_scan = self.table[idx]["norm_scan"]
-            dloglike_scan = (
-                self.table[idx]["dloglike_scan"] - self.table[idx]["loglike"]
-            )
-            z[idx] = _interp_likelihood_profile(norm_scan, dloglike_scan, norm)
+            norm_scan = row["norm_scan"]
+            dloglike_scan = row["dloglike_scan"] - row["loglike"]
+            interp = interpolate_likelihood_profile(norm_scan, dloglike_scan)
+            z[idx] = interp((norm,))
 
         kwargs.setdefault("vmax", 0)
         kwargs.setdefault("vmin", -4)
@@ -744,11 +730,11 @@ class FluxPoints(object):
         return ax
 
 
-class FluxPointEstimator(object):
-    """Flux point estimator.
+class FluxPointsEstimator:
+    """Flux points estimator.
 
-    Estimates flux points for a given spectrum observation dataset, energy groups
-    and spectral model.
+    Estimates flux points for a given list of spectral datasets, energies and
+    spectral model.
 
     To estimate the flux point the amplitude of the reference spectral model is
     fitted within the energy range defined by the energy group. This is done for
@@ -757,19 +743,19 @@ class FluxPointEstimator(object):
     energy group. See https://gamma-astro-data-formats.readthedocs.io/en/latest/spectra/binned_likelihoods/index.html
     for details.
 
-    The method is also described in the FERMI-LAT catalog paper
+    The method is also described in the Fermi-LAT catalog paper
     https://ui.adsabs.harvard.edu/#abs/2015ApJS..218...23A
     or the HESS Galactic Plane Survey paper
     https://ui.adsabs.harvard.edu/#abs/2018A%26A...612A...1H
 
     Parameters
     ----------
-    obs : `~gammapy.spectrum.SpectrumObservation` or `~gammapy.spectrum.SpectrumObservationList`
-        Spectrum observation(s)
-    groups : `~gammapy.spectrum.SpectrumEnergyGroups`
-        Energy groups (usually output of `~gammapy.spectrum.SpectrumEnergyGroupMaker`)
-    model : `~gammapy.spectrum.models.SpectralModel`
-        Global model (usually output of `~gammapy.spectrum.SpectrumFit`)
+    datasets : list of `~gammapy.spectrum.SpectrumDatatset`
+        Spectrum datasets.
+    e_edges : `~astropy.units.Quantity`
+        Energy edges of the flux point bins.
+    source : str
+        For which source in the model to compute the flux points.
     norm_min : float
         Minimum value for the norm used for the likelihood profile evaluation.
     norm_max : float
@@ -782,43 +768,103 @@ class FluxPointEstimator(object):
         Sigma to use for asymmetric error computation.
     sigma_ul : int
         Sigma to use for upper limit computation.
+    reoptimize : bool
+        Re-optimize other free model parameters.
     """
 
     def __init__(
         self,
-        obs,
-        groups,
-        model,
+        datasets,
+        e_edges,
+        source="",
         norm_min=0.2,
         norm_max=5,
         norm_n_values=11,
         norm_values=None,
         sigma=1,
         sigma_ul=2,
+        reoptimize=False,
     ):
-        if isinstance(obs, SpectrumObservation):
-            obs = SpectrumObservationList([obs])
-        self.obs = SpectrumObservationList(obs)
+        # make a copy to not modify the input datasets
+        if not isinstance(datasets, Datasets):
+            datasets = Datasets(datasets)
+
+        if not datasets.is_all_same_type and datasets.is_all_same_shape:
+            raise ValueError(
+                "Flux point estimation requires a list of datasets"
+                " of the same type and data shape."
+            )
+
+        self.datasets = datasets.copy()
+        self.e_edges = e_edges
+
+        dataset = self.datasets.datasets[0]
+
+        if isinstance(dataset, SpectrumDatasetOnOff):
+            model = dataset.model
+        else:
+            model = dataset.model[source].spectral_model
 
-        self.groups = groups
         self.model = ScaleModel(model)
-        self.model.parameters["norm"].min = 0
+        self.model.norm.min = 0
+        self.model.norm.max = 1e3
+
         if norm_values is None:
             norm_values = np.logspace(
                 np.log10(norm_min), np.log10(norm_max), norm_n_values
             )
+
         self.norm_values = norm_values
         self.sigma = sigma
         self.sigma_ul = sigma_ul
+        self.reoptimize = reoptimize
+        self.source = source
+        self.fit = Fit(self.datasets)
+
+        self._set_scale_model()
+
+    def _freeze_parameters(self):
+        # freeze other parameters
+        for par in self.datasets.parameters:
+            if par is not self.model.norm:
+                par.frozen = True
+
+    def _freeze_empty_background(self):
+        from ..cube import MapDataset
+
+        counts_all = self.estimate_counts()["counts"]
+
+        for counts, dataset in zip(counts_all, self.datasets.datasets):
+            if isinstance(dataset, MapDataset) and counts == 0:
+                if dataset.background_model is not None:
+                    dataset.background_model.parameters.freeze_all()
+
+    def _set_scale_model(self):
+        # set the model on all datasets
+        for dataset in self.datasets.datasets:
+            if isinstance(dataset, SpectrumDatasetOnOff):
+                dataset.model = self.model
+            else:
+                dataset.model[self.source].spectral_model = self.model
 
     @property
     def ref_model(self):
         return self.model.model
 
+    @property
+    def e_groups(self):
+        """Energy grouping table `~astropy.table.Table`"""
+        dataset = self.datasets.datasets[0]
+        if isinstance(dataset, SpectrumDatasetOnOff):
+            energy_axis = dataset.counts.energy
+        else:
+            energy_axis = dataset.counts.geom.get_axis_by_name("energy")
+        return energy_axis.group_table(self.e_edges)
+
     def __str__(self):
         s = "{}:\n".format(self.__class__.__name__)
-        s += str(self.obs) + "\n"
-        s += str(self.groups) + "\n"
+        s += str(self.datasets) + "\n"
+        s += str(self.e_edges) + "\n"
         s += str(self.model) + "\n"
         return s
 
@@ -834,24 +880,30 @@ class FluxPointEstimator(object):
             and available options.
         """
         rows = []
-        for group in self.groups:
-            if group.bin_type != "normal":
-                log.debug("Skipping energy group:\n{}".format(group))
+
+        for e_group in self.e_groups:
+            if e_group["bin_type"].strip() != "normal":
+                log.debug("Skipping under-/ overflow bin in flux point estimation.")
                 continue
 
-            row = self.estimate_flux_point(group, steps=steps)
+            row = self.estimate_flux_point(e_group, steps=steps)
             rows.append(row)
 
         meta = OrderedDict([("SED_TYPE", "likelihood")])
         table = table_from_row_data(rows=rows, meta=meta)
         return FluxPoints(table).to_sed_type("dnde")
 
+    def _energy_mask(self, e_group):
+        energy_mask = np.zeros(self.datasets.datasets[0].data_shape)
+        energy_mask[e_group["idx_min"] : e_group["idx_max"] + 1] = 1
+        return energy_mask.astype(bool)
+
     def estimate_flux_point(self, e_group, steps="all"):
         """Estimate flux point for a single energy group.
 
         Parameters
         ----------
-        e_group : `SpectrumEnergyGroup`
+        e_group : `~astropy.table.Row`
             Energy group to compute the flux point for.
         steps : list of str
             Which steps to execute. Available options are:
@@ -869,8 +921,8 @@ class FluxPointEstimator(object):
         result : dict
             Dict with results for the flux point.
         """
+        e_min, e_max = e_group["energy_min"], e_group["energy_max"]
         # Put at log center of the bin
-        e_min, e_max = e_group.energy_min, e_group.energy_max
         e_ref = np.sqrt(e_min * e_max)
 
         result = OrderedDict(
@@ -885,28 +937,62 @@ class FluxPointEstimator(object):
             ]
         )
 
-        quality_orig = self._set_quality(e_group)
-        result.update(self.estimate_norm())
+        contribute_to_likelihood = False
 
-        if steps == "all":
-            steps = ["err", "errp-errn", "ul", "ts", "norm-scan"]
+        for dataset in self.datasets.datasets:
+            dataset.mask_fit = self._energy_mask(e_group)
+            mask = dataset.mask_fit
 
-        if "err" in steps:
-            result.update(self.estimate_norm_err())
+            if dataset.mask_safe is not None:
+                mask &= dataset.mask_safe
 
-        if "errp-errn" in steps:
-            result.update(self.estimate_norm_errn_errp())
+            contribute_to_likelihood |= mask.any()
 
-        if "ul" in steps:
-            result.update(self.estimate_norm_ul())
+        if not contribute_to_likelihood:
+            raise ValueError(
+                "No dataset contributes to the likelihood between"
+                " {e_min:.3f} and {e_max:.3f}. Please adapt the "
+                "flux point energy edges or check the dataset masks.".format(
+                    e_min=e_min, e_max=e_max
+                )
+            )
 
-        if "ts" in steps:
-            result.update(self.estimate_norm_ts())
+        with self.datasets.parameters.restore_values:
 
-        if "norm-scan" in steps:
-            result.update(self.estimate_norm_scan(result))
+            self._freeze_empty_background()
+
+            if not self.reoptimize:
+                self._freeze_parameters()
+
+            result.update(self.estimate_norm())
+
+            if not result.pop("success"):
+                log.warning(
+                    "Fit failed for flux point between {e_min:.3f} and {e_max:.3f},"
+                    " setting NaN.".format(e_min=e_min, e_max=e_max)
+                )
+
+            if steps == "all":
+                steps = ["err", "counts", "errp-errn", "ul", "ts", "norm-scan"]
+
+            if "err" in steps:
+                result.update(self.estimate_norm_err())
+
+            if "counts" in steps:
+                result.update(self.estimate_counts())
+
+            if "errp-errn" in steps:
+                result.update(self.estimate_norm_errn_errp())
+
+            if "ul" in steps:
+                result.update(self.estimate_norm_ul())
+
+            if "ts" in steps:
+                result.update(self.estimate_norm_ts())
+
+            if "norm-scan" in steps:
+                result.update(self.estimate_norm_scan())
 
-        self._restore_quality(quality_orig)
         return result
 
     def estimate_norm_errn_errp(self):
@@ -917,7 +1003,7 @@ class FluxPointEstimator(object):
         result : dict
             Dict with asymmetric errors for the flux point norm.
         """
-        result = self.fit.confidence(parameter="norm", sigma=self.sigma)
+        result = self.fit.confidence(parameter=self.model.norm, sigma=self.sigma)
         return {"norm_errp": result["errp"], "norm_errn": result["errn"]}
 
     def estimate_norm_err(self):
@@ -929,9 +1015,32 @@ class FluxPointEstimator(object):
             Dict with symmetric error for the flux point norm.
         """
         result = self.fit.covariance()
-        norm_err = result.model.parameters.error("norm")
+        norm_err = result.parameters.error(self.model.norm)
         return {"norm_err": norm_err}
 
+    def estimate_counts(self):
+        """Estimate counts for the flux point
+
+        Returns
+        -------
+        result : dict
+            Dict with an array with one entry per dataset with counts for the flux point.
+        """
+
+        counts = []
+
+        for dataset in self.datasets.datasets:
+            mask = dataset.mask_fit
+            if dataset.mask_safe is not None:
+                mask &= dataset.mask_safe
+
+            if isinstance(dataset, SpectrumDatasetOnOff):
+                counts.append(dataset.counts.data.data[mask].sum())
+            else:
+                counts.append(dataset.counts.data[mask].sum())
+
+        return {"counts": np.array(counts, dtype=int)}
+
     def estimate_norm_ul(self):
         """Estimate upper limit for a flux point.
 
@@ -940,9 +1049,24 @@ class FluxPointEstimator(object):
         result : dict
             Dict with upper limit for the flux point norm.
         """
-        norm = self.model.parameters["norm"].value
-        result = self.fit.confidence(parameter="norm", sigma=self.sigma_ul)
-        return {"norm_ul": result["errp"] + norm}
+        norm = self.model.norm
+
+        # TODO: the minuit backend has convergence problems when the likelihood is not
+        #  of parabolic shape, which is the case, when there are zero counts in the
+        #  energy bin. For this case we change to the scipy backend.
+        counts = self.estimate_counts()["counts"]
+
+        if np.all(counts == 0):
+            result = self.fit.confidence(
+                parameter=norm,
+                sigma=self.sigma_ul,
+                backend="scipy",
+                reoptimize=self.reoptimize,
+            )
+        else:
+            result = self.fit.confidence(parameter=norm, sigma=self.sigma_ul)
+
+        return {"norm_ul": result["errp"] + norm.value}
 
     def estimate_norm_ts(self):
         """Estimate ts and sqrt(ts) for the flux point.
@@ -950,24 +1074,25 @@ class FluxPointEstimator(object):
         Returns
         -------
         result : dict
-            Dict with ts and srtq_ts for the flux point.
+            Dict with ts and sqrt(ts) for the flux point.
         """
-        parameters = self.model.parameters
-
-        loglike = self.fit.total_stat(parameters)
-        norm_best_fit = parameters["norm"].value
+        loglike = self.datasets.likelihood()
 
         # store best fit amplitude, set amplitude of fit model to zero
-        parameters["norm"].value = 0
-        loglike_null = self.fit.total_stat(parameters)
-        parameters["norm"].value = 1.0
+        self.model.norm.value = 0
+        self.model.norm.frozen = True
+
+        if self.reoptimize:
+            _ = self.fit.optimize()
+
+        loglike_null = self.datasets.likelihood()
 
         # compute sqrt TS
         ts = np.abs(loglike_null - loglike)
-        sqrt_ts = np.sign(norm_best_fit) * np.sqrt(ts)
+        sqrt_ts = np.sqrt(ts)
         return {"sqrt_ts": sqrt_ts, "ts": ts}
 
-    def estimate_norm_scan(self, flux_point):
+    def estimate_norm_scan(self):
         """Estimate likelihood profile for the norm parameter
 
         Returns
@@ -975,8 +1100,9 @@ class FluxPointEstimator(object):
         result : dict
             Dict with norm_scan and dloglike_scan for the flux point.
         """
-        # norm = self.model.parameters["norm"]
-        result = self.fit.likelihood_profile("norm", values=self.norm_values)
+        result = self.fit.likelihood_profile(
+            self.model.norm, values=self.norm_values, reoptimize=self.reoptimize
+        )
         dloglike_scan = result["likelihood"]
         return {"norm_scan": result["values"], "dloglike_scan": dloglike_scan}
 
@@ -988,51 +1114,21 @@ class FluxPointEstimator(object):
         result : dict
             Dict with "norm" and "loglike" for the flux point.
         """
-        from .fit import SpectrumFit
+        # start optimization with norm=1
+        self.model.norm.value = 1.0
+        self.model.norm.frozen = False
 
-        self.fit = SpectrumFit(self.obs, self.model)
         result = self.fit.optimize()
 
         if result.success:
-            norm = result.model.parameters["norm"].value
-            self.model.parameters["norm"].value = norm
+            norm = self.model.norm.value
         else:
-            emin, emax = self.fit.true_fit_range[0]
-            log.warning(
-                "Fit failed for flux point between {emin:.3f} and {emax:.3f},"
-                " setting NaN.".format(emin=emin, emax=emax)
-            )
             norm = np.nan
 
-        return {"norm": norm, "loglike": result.total_stat}
-
-    # TODO: clean up this helper function and maybe move it to SpectrumObservations
-    def _set_quality(self, energy_group):
-        quality_orig = []
-
-        for index in range(len(self.obs)):
-            # Set quality bins to only bins in energy_group
-            quality_orig_unit = self.obs[index].on_vector.quality
-            quality_len = len(quality_orig_unit)
-            quality_orig.append(quality_orig_unit)
-            quality = np.zeros(quality_len, dtype=int)
-            for bin in range(quality_len):
-                if (
-                    (bin < energy_group.bin_idx_min)
-                    or (bin > energy_group.bin_idx_max)
-                    or (energy_group.bin_type != "normal")
-                ):
-                    quality[bin] = 1
-            self.obs[index].on_vector.quality = quality
-
-        return quality_orig
-
-    def _restore_quality(self, quality_orig):
-        for index in range(len(quality_orig)):
-            self.obs[index].on_vector.quality = quality_orig[index]
+        return {"norm": norm, "loglike": result.total_stat, "success": result.success}
 
 
-class FluxPointFit(Fit):
+class FluxPointsDataset(Dataset):
     """
     Fit a set of flux points with a parametric model.
 
@@ -1042,14 +1138,20 @@ class FluxPointFit(Fit):
         Spectral model
     data : `~gammapy.spectrum.FluxPoints`
         Flux points.
+    mask_fit : `numpy.ndarray`
+        Mask to apply to the likelihood for fitting.
+    likelihood : {"chi2", "chi2assym"}
+        Likelihood function to use for the fit.
+    mask_safe : `numpy.ndarray`
+        Mask defining the safe data range.
 
     Examples
     --------
-
     Load flux points from file and fit with a power-law model::
 
         from astropy import units as u
-        from gammapy.spectrum import FluxPoints, FluxPointFit
+        from gammapy.spectrum import FluxPoints, FluxPointsDataset
+        from gammapy.utils.fitting import Fit
         from gammapy.spectrum.models import PowerLaw
 
         filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits'
@@ -1057,53 +1159,213 @@ class FluxPointFit(Fit):
 
         model = PowerLaw()
 
-        fit = FluxPointFit(model, flux_points)
+        dataset = FluxPointsDataset(model, flux_points)
+        fit = Fit(dataset)
         result = fit.run()
         print(result)
-        print(result.model)
+        print(result.parameters.to_table())
     """
 
-    def __init__(self, model, data, stat="chi2"):
-        self._model = model
+    def __init__(self, model, data, mask_fit=None, likelihood="chi2", mask_safe=None):
+        self.model = model
         self.data = data
+        self.mask_fit = mask_fit
+        self.parameters = model.parameters
 
-        if stat in ["chi2", "chi2assym"]:
-            self._stat = stat
+        if data.sed_type != "dnde":
+            raise ValueError("Currently only flux points of type 'dnde' are supported.")
+
+        if mask_safe is None:
+            mask_safe = np.isfinite(data.table["dnde"])
+
+        self.mask_safe = mask_safe
+
+        if likelihood in ["chi2", "chi2assym"]:
+            self._likelihood = likelihood
         else:
             raise ValueError(
-                "'{stat}' is not a valid fit statistic, please choose"
+                "'{likelihood}' is not a valid fit statistic, please choose"
                 " either 'chi2' or 'chi2assym'"
             )
 
-    @property
-    def _stat_chi2(self):
-        """Likelihood per bin given the current model parameters"""
-        model = self._model(self.data.e_ref)
-        data = self.data.table["dnde"].quantity
-        sigma = self.data.table["dnde_err"].quantity
+    def data_shape(self):
+        """Shape of the flux points data"""
+        return self.data.e_ref.shape
+
+    @staticmethod
+    def _likelihood_chi2(data, model, sigma):
         return ((data - model) / sigma).to_value("") ** 2
 
-    @property
-    def _stat_chi2_assym(self):
+    @staticmethod
+    def _likelihood_chi2_assym(data, model, sigma_n, sigma_p):
         """
         Assymetric chi2 statistics for a list of flux points and model.
         """
-        model = self._model(self.data.e_ref)
-        data = self.data.table["dnde"].quantity.to(model.unit)
-        data_errp = self.data.table["dnde_errp"].quantity
-        data_errn = self.data.table["dnde_errn"].quantity
+        is_p = model > data
+        sigma = sigma_n
+        sigma[is_p] = sigma_p[is_p]
+        return FluxPointsDataset._likelihood_chi2(data, model, sigma)
 
-        val_n = ((data - model) / data_errn).to_value("") ** 2
-        val_p = ((data - model) / data_errp).to_value("") ** 2
-        return np.where(model > data, val_p, val_n)
+    def flux_pred(self):
+        """Compute predicted flux."""
+        return self.model(self.data.e_ref)
+
+    def likelihood_per_bin(self):
+        """Likelihood per bin given the current model parameters"""
+        model = self.flux_pred()
+        data = self.data.table["dnde"].quantity
+
+        if self._likelihood == "chi2":
+            sigma = self.data.table["dnde_err"].quantity
+            return self._likelihood_chi2(data, model, sigma)
+        elif self._likelihood == "chi2assym":
+            sigma_n = self.data.table["dnde_errn"].quantity
+            sigma_p = self.data.table["dnde_errp"].quantity
+            return self._likelihood_chi2_assym(data, model, sigma_n, sigma_p)
+        else:
+            # TODO: add likelihood profiles
+            pass
+
+    def residuals(self):
+        """Compute flux point residuals
+
+        Defined as `(data - model) / model`.
+
+        Returns
+        -------
+        residuals : `~numpy.ndarray`
+            Flux point residuals
+        """
+        fp = self.data
+        data = fp.table[fp.sed_type].quantity
+
+        model = self.model(fp.e_ref)
+        residuals = ((data - model) / model).to_value("")
+
+        # Remove residuals for upper_limits
+        residuals[fp.is_ul] = np.nan
+        return residuals
+
+    def peek(self, **kwargs):
+        """Plot flux points, best fit model and residuals.
+        """
+        from matplotlib.gridspec import GridSpec
+        import matplotlib.pyplot as plt
+
+        gs = GridSpec(7, 1)
+
+        ax_spectrum = plt.subplot(gs[:5, :])
+        self.plot_spectrum(ax=ax_spectrum, **kwargs)
+
+        ax_spectrum.set_xticks([])
+
+        ax_residuals = plt.subplot(gs[5:, :])
+        self.plot_residuals(ax=ax_residuals)
+        return ax_spectrum, ax_residuals
 
     @property
-    def stat(self):
-        if self._stat == "chi2":
-            return self._stat_chi2
-        else:
-            return self._stat_chi2_assym
+    def _e_range(self):
+        try:
+            return u.Quantity([self.data.e_min.min(), self.data.e_max.max()])
+        except KeyError:
+            return u.Quantity([self.data.e_ref.min(), self.data.e_ref.max()])
 
-    def total_stat(self, parameters):
-        """Total likelihood given the current model parameters"""
-        return np.nansum(self.stat, dtype=np.float64)
+    @property
+    def _e_unit(self):
+        return self.data.e_ref.unit
+
+    def plot_residuals(self, ax=None, **kwargs):
+        """Plot flux point residuals.
+
+        Parameters
+        ----------
+        ax : `~matplotlib.pyplot.Axes`
+            Axes object.
+        **kwargs : dict
+            Keyword arguments passed to `~matplotlib.pyplot.errorbar`.
+
+        Returns
+        -------
+        ax : `~matplotlib.pyplot.Axes`
+            Axes object.
+        """
+        import matplotlib.pyplot as plt
+
+        ax = plt.gca() if ax is None else ax
+
+        residuals = self.residuals()
+
+        fp = self.data
+
+        xerr = fp._plot_get_energy_err()
+        if xerr is not None:
+            xerr = xerr[0].to_value(self._e_unit), xerr[1].to_value(self._e_unit)
+
+        model = self.model(fp.e_ref)
+        yerr = fp._plot_get_flux_err(fp.sed_type)
+        yerr = (yerr[0] / model).to_value(""), (yerr[1] / model).to_value("")
+
+        kwargs.setdefault("marker", "+")
+        kwargs.setdefault("ls", "None")
+        ax.errorbar(self.data.e_ref.value, residuals, xerr=xerr, yerr=yerr, **kwargs)
+
+        # format axes
+        ax.axhline(0, color="black", lw=0.5)
+        ax.set_ylabel("Residuals")
+        ax.set_xlabel("Energy ({})".format(self._e_unit))
+        ax.set_ylim(-1, 1)
+        ax.set_xscale("log")
+        ax.set_xlim(self._e_range.to_value(self._e_unit))
+        return ax
+
+    def plot_spectrum(self, ax=None, fp_kwargs=None, model_kwargs=None):
+        """
+        Plot spectrum including flux points and model.
+
+        Parameters
+        ----------
+        ax : `~matplotlib.pyplot.Axes`
+            Axes object.
+        fp_kwargs : dict
+            Keyword arguments passed to `FluxPoints.plot`.
+        model_kwargs : dict
+            Keywords passed to `SpectralModel.plot` and `SpectralModel.plot_error`
+
+        Returns
+        -------
+        ax : `~matplotlib.pyplot.Axes`
+            Axes object.
+        """
+        import matplotlib.pyplot as plt
+
+        ax = plt.gca() if ax is None else ax
+        fp_kwargs = {} if fp_kwargs is None else fp_kwargs
+        model_kwargs = {} if model_kwargs is None else model_kwargs
+
+        kwargs = {}
+        kwargs.setdefault("flux_unit", "erg-1 cm-2 s-1")
+        kwargs.setdefault("energy_unit", "TeV")
+        kwargs.setdefault("energy_power", 2)
+
+        # plot flux points
+        plot_kwargs = kwargs.copy()
+        plot_kwargs.update(fp_kwargs)
+        plot_kwargs.setdefault("label", "Flux points")
+        ax = self.data.plot(ax=ax, **plot_kwargs)
+
+        plot_kwargs = kwargs.copy()
+        plot_kwargs.setdefault("energy_range", self._e_range)
+        plot_kwargs.setdefault("zorder", 10)
+        plot_kwargs.update(model_kwargs)
+        plot_kwargs.setdefault("label", "Best fit model")
+        self.model.plot(ax=ax, **plot_kwargs)
+
+        plot_kwargs.setdefault("color", ax.lines[-1].get_color())
+        del plot_kwargs["label"]
+
+        if self.model.parameters.covariance is not None:
+            self.model.plot_error(ax=ax, **plot_kwargs)
+
+        # format axes
+        ax.set_xlim(self._e_range.to_value(self._e_unit))
+        return ax
