@@ -1,13 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Spectral models for Gammapy."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import operator
 import numpy as np
 from scipy.optimize import brentq
 import astropy.units as u
 from astropy.table import Table
 from ..utils.energy import EnergyBounds
-from ..utils.nddata import NDDataArray, BinnedDataAxis
 from ..utils.scripts import make_path
 from ..utils.fitting import Parameter, Parameters, Model
 from ..utils.interpolation import ScaledRegularGridInterpolator
@@ -26,6 +24,7 @@ __all__ = [
     "TableModel",
     "AbsorbedSpectralModel",
     "Absorption",
+    "NaimaModel",
 ]
 
 
@@ -219,8 +218,9 @@ class SpectralModel(Model):
     @classmethod
     def from_dict(cls, val):
         """Create from dict."""
-        classname = val.pop("name")
-        parameters = Parameters.from_dict(val)
+        val_copy = val.copy()
+        classname = val_copy.pop("name")
+        parameters = Parameters.from_dict(val_copy)
         model = globals()[classname]()
         model.parameters = parameters
         model.parameters.covariance = parameters.covariance
@@ -358,7 +358,8 @@ class SpectralModel(Model):
         self._plot_format_ax(ax, energy, y_lo, energy_power)
         return ax
 
-    def _plot_format_ax(self, ax, energy, y, energy_power):
+    @staticmethod
+    def _plot_format_ax(ax, energy, y, energy_power):
         ax.set_xlabel("Energy [{}]".format(energy.unit))
         if energy_power > 0:
             ax.set_ylabel("E{} * Flux [{}]".format(energy_power, y.unit))
@@ -368,7 +369,8 @@ class SpectralModel(Model):
         ax.set_xscale("log", nonposx="clip")
         ax.set_yscale("log", nonposy="clip")
 
-    def _plot_scale_flux(self, energy, flux, energy_power):
+    @staticmethod
+    def _plot_scale_flux(energy, flux, energy_power):
         try:
             eunit = [_ for _ in flux.unit.bases if _.physical_type == "energy"][0]
         except IndexError:
@@ -434,9 +436,7 @@ class SpectralModel(Model):
 class ConstantModel(SpectralModel):
     r"""Constant model
 
-    .. math::
-
-        \phi(E) = k
+    .. math:: \phi(E) = k
 
     Parameters
     ----------
@@ -444,8 +444,12 @@ class ConstantModel(SpectralModel):
         :math:`k`
     """
 
+    __slots__ = ["const"]
+
     def __init__(self, const):
-        self.parameters = Parameters([Parameter("const", const)])
+        self.const = Parameter("const", const)
+
+        super().__init__([self.const])
 
     @staticmethod
     def evaluate(energy, const):
@@ -463,18 +467,12 @@ class CompoundSpectralModel(SpectralModel):
         self.model1 = model1
         self.model2 = model2
         self.operator = operator
+        parameters = (
+            self.model1.parameters.parameters + self.model2.parameters.parameters
+        )
+        super().__init__(parameters)
 
     # TODO: Think about how to deal with covariance matrix
-    @property
-    def parameters(self):
-        val = self.model1.parameters.parameters + self.model2.parameters.parameters
-        return Parameters(val)
-
-    @parameters.setter
-    def parameters(self, parameters):
-        idx = len(self.model1.parameters.parameters)
-        self.model1.parameters.parameters = parameters.parameters[:idx]
-        self.model2.parameters.parameters = parameters.parameters[idx:]
 
     def __str__(self):
         ss = self.__class__.__name__
@@ -523,14 +521,14 @@ class PowerLaw(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["index", "amplitude", "reference"]
+
     def __init__(self, index=2.0, amplitude="1e-12 cm-2 s-1 TeV-1", reference="1 TeV"):
-        self.parameters = Parameters(
-            [
-                Parameter("index", index),
-                Parameter("amplitude", amplitude),
-                Parameter("reference", reference, frozen=True),
-            ]
-        )
+        self.index = Parameter("index", index)
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.reference = Parameter("reference", reference, frozen=True)
+
+        super().__init__([self.index, self.amplitude, self.reference])
 
     @staticmethod
     def evaluate(energy, index, amplitude, reference):
@@ -711,17 +709,17 @@ class PowerLaw2(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["index", "amplitude", "emin", "emax"]
+
     def __init__(
         self, amplitude="1e-12 cm-2 s-1", index=2, emin="0.1 TeV", emax="100 TeV"
     ):
-        self.parameters = Parameters(
-            [
-                Parameter("amplitude", amplitude),
-                Parameter("index", index),
-                Parameter("emin", emin, frozen=True),
-                Parameter("emax", emax, frozen=True),
-            ]
-        )
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.index = Parameter("index", index)
+        self.emin = Parameter("emin", emin, frozen=True)
+        self.emax = Parameter("emax", emax, frozen=True)
+
+        super().__init__([self.index, self.amplitude, self.emin, self.emax])
 
     @staticmethod
     def evaluate(energy, amplitude, index, emin, emax):
@@ -839,6 +837,8 @@ class ExponentialCutoffPowerLaw(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["index", "amplitude", "reference", "lambda_"]
+
     def __init__(
         self,
         index=1.5,
@@ -846,14 +846,12 @@ class ExponentialCutoffPowerLaw(SpectralModel):
         reference="1 TeV",
         lambda_="0.1 TeV-1",
     ):
-        self.parameters = Parameters(
-            [
-                Parameter("index", index),
-                Parameter("amplitude", amplitude),
-                Parameter("reference", reference, frozen=True),
-                Parameter("lambda_", lambda_),
-            ]
-        )
+        self.index = Parameter("index", index)
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.reference = Parameter("reference", reference, frozen=True)
+        self.lambda_ = Parameter("lambda_", lambda_)
+
+        super().__init__([self.index, self.amplitude, self.reference, self.lambda_])
 
     @staticmethod
     def evaluate(energy, index, amplitude, reference, lambda_):
@@ -918,6 +916,8 @@ class ExponentialCutoffPowerLaw3FGL(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["index", "amplitude", "reference", "ecut"]
+
     def __init__(
         self,
         index=1.5,
@@ -925,14 +925,12 @@ class ExponentialCutoffPowerLaw3FGL(SpectralModel):
         reference="1 TeV",
         ecut="10 TeV",
     ):
-        self.parameters = Parameters(
-            [
-                Parameter("index", index),
-                Parameter("amplitude", amplitude),
-                Parameter("reference", reference, frozen=True),
-                Parameter("ecut", ecut),
-            ]
-        )
+        self.index = Parameter("index", index)
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.reference = Parameter("reference", reference, frozen=True)
+        self.ecut = Parameter("ecut", ecut)
+
+        super().__init__([self.index, self.amplitude, self.reference, self.ecut])
 
     @staticmethod
     def evaluate(energy, index, amplitude, reference, ecut):
@@ -981,6 +979,8 @@ class PLSuperExpCutoff3FGL(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["index_1", "index_2", "amplitude", "reference", "ecut"]
+
     def __init__(
         self,
         index_1=1.5,
@@ -989,14 +989,14 @@ class PLSuperExpCutoff3FGL(SpectralModel):
         reference="1 TeV",
         ecut="10 TeV",
     ):
-        self.parameters = Parameters(
-            [
-                Parameter("index_1", index_1),
-                Parameter("index_2", index_2),
-                Parameter("amplitude", amplitude),
-                Parameter("reference", reference, frozen=True),
-                Parameter("ecut", ecut),
-            ]
+        self.index_1 = Parameter("index_1", index_1)
+        self.index_2 = Parameter("index_2", index_2)
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.reference = Parameter("reference", reference, frozen=True)
+        self.ecut = Parameter("ecut", ecut)
+
+        super().__init__(
+            [self.index_1, self.index_2, self.amplitude, self.reference, self.ecut]
         )
 
     @staticmethod
@@ -1053,17 +1053,17 @@ class LogParabola(SpectralModel):
         plt.show()
     """
 
+    __slots__ = ["amplitude", "reference", "alpha", "beta"]
+
     def __init__(
         self, amplitude="1e-12 cm-2 s-1 TeV-1", reference="10 TeV", alpha=2, beta=1
     ):
-        self.parameters = Parameters(
-            [
-                Parameter("amplitude", amplitude),
-                Parameter("reference", reference, frozen=True),
-                Parameter("alpha", alpha),
-                Parameter("beta", beta),
-            ]
-        )
+        self.amplitude = Parameter("amplitude", amplitude)
+        self.reference = Parameter("reference", reference, frozen=True)
+        self.alpha = Parameter("alpha", alpha)
+        self.beta = Parameter("beta", beta)
+
+        super().__init__([self.amplitude, self.reference, self.alpha, self.beta])
 
     @classmethod
     def from_log10(cls, amplitude, reference, alpha, beta):
@@ -1131,21 +1131,26 @@ class TableModel(SpectralModel):
         Meta information, meta['filename'] will be used for serialization
     """
 
+    __slots__ = ["energy", "values", "norm", "meta", "_evaluate"]
+
     def __init__(
         self, energy, values, norm=1, values_scale="log", interp_kwargs=None, meta=None
     ):
-        self.parameters = Parameters([Parameter("norm", norm, unit="")])
+        self.norm = Parameter("norm", norm, unit="")
         self.energy = energy
         self.values = values
         self.meta = dict() if meta is None else meta
 
         interp_kwargs = interp_kwargs or {}
         interp_kwargs.setdefault("values_scale", "log")
+        interp_kwargs.setdefault("points_scale", ("log",))
 
         self._evaluate = ScaledRegularGridInterpolator(
-            points=(np.log(energy.value),), values=values, **interp_kwargs
+            points=(energy,), values=values, **interp_kwargs
         )
 
+        super().__init__([self.norm])
+
     @classmethod
     def read_xspec_model(cls, filename, param, **kwargs):
         """Read XSPEC table model
@@ -1221,8 +1226,7 @@ class TableModel(SpectralModel):
 
     def evaluate(self, energy, norm):
         """Evaluate the model (static function)."""
-        x = np.log(energy.to_value(self.energy.unit))
-        values = self._evaluate((x,), clip=True)
+        values = self._evaluate((energy,), clip=True)
         return norm * values
 
 
@@ -1237,15 +1241,18 @@ class ScaleModel(SpectralModel):
         Multiplicative norm factor for the model value.
     """
 
+    __slots__ = ["norm", "model"]
+
     def __init__(self, model, norm=1):
-        self.parameters = Parameters([Parameter("norm", norm, unit="")])
+        self.norm = Parameter("norm", norm, unit="")
         self.model = model
+        super().__init__([self.norm])
 
     def evaluate(self, energy, norm):
         return norm * self.model(energy)
 
 
-class Absorption(object):
+class Absorption:
     r"""Gamma-ray absorption models.
 
     Parameters
@@ -1295,18 +1302,21 @@ class Absorption(object):
         plt.show()
     """
 
-    def __init__(self, energy_lo, energy_hi, param_lo, param_hi, data):
-        axes = [
-            BinnedDataAxis(
-                param_lo, param_hi, interpolation_mode="linear", name="parameter"
-            ),
-            BinnedDataAxis(
-                energy_lo, energy_hi, interpolation_mode="log", name="energy"
-            ),
-        ]
+    def __init__(
+        self, energy_lo, energy_hi, param_lo, param_hi, data, interp_kwargs=None
+    ):
+        self.data = data
 
-        self.data = NDDataArray(axes=axes, data=data)
-        self.data.default_interp_kwargs["fill_value"] = None
+        # set values log centers
+        self.energy = np.sqrt(energy_lo * energy_hi)
+        self.param = (param_hi + param_lo) / 2
+
+        interp_kwargs = interp_kwargs or {}
+        interp_kwargs.setdefault("points_scale", ("log", "lin"))
+
+        self._evaluate = ScaledRegularGridInterpolator(
+            points=(self.param, self.energy), values=data, **interp_kwargs
+        )
 
     @classmethod
     def read(cls, filename):
@@ -1391,16 +1401,13 @@ class Absorption(object):
         unit : str, (optional)
             desired value for energy axis
         """
-        energy_axis = self.data.axes[1]
-        energy = (energy_axis.log_center()).to(unit)
-
+        energy = self.energy.to(unit)
         values = self.evaluate(energy=energy, parameter=parameter)
-
         return TableModel(energy=energy, values=values, values_scale="lin")
 
     def evaluate(self, energy, parameter):
         """Evaluate model for energy and parameter value."""
-        return self.data.evaluate(energy=energy, parameter=parameter)
+        return self._evaluate((parameter, energy))
 
 
 class AbsorbedSpectralModel(SpectralModel):
@@ -1418,6 +1425,8 @@ class AbsorbedSpectralModel(SpectralModel):
         parameter name
     """
 
+    __slots__ = ["spectral_model", "absorption", "parameter", "parameter_name"]
+
     def __init__(
         self, spectral_model, absorption, parameter, parameter_name="redshift"
     ):
@@ -1426,18 +1435,14 @@ class AbsorbedSpectralModel(SpectralModel):
         self.parameter = parameter
         self.parameter_name = parameter_name
 
-        # initialise list parameters from spectral model
-        param_list = []
-        for param in spectral_model.parameters.parameters:
-            param_list.append(param)
-
-        # Add parameter to the list
-        min_ = self.absorption.data.axes[0].lo[0]
-        max_ = self.absorption.data.axes[0].lo[-1]
+        min_ = self.absorption.param.min()
+        max_ = self.absorption.param.max()
         par = Parameter(parameter_name, parameter, min=min_, max=max_, frozen=True)
-        param_list.append(par)
 
-        self.parameters = Parameters(param_list)
+        parameters = spectral_model.parameters.parameters.copy()
+        parameters.append(par)
+
+        super().__init__(parameters)
 
     def evaluate(self, energy, **kwargs):
         """Evaluate the model at a given energy."""
@@ -1449,3 +1454,135 @@ class AbsorbedSpectralModel(SpectralModel):
         flux = self.spectral_model.evaluate(energy=energy, **kwargs)
         absorption = self.absorption.evaluate(energy=energy, parameter=parameter)
         return flux * absorption
+
+
+class NaimaModel(SpectralModel):
+    r"""A wrapper for Naima models
+
+    This class provides an interface with the models defined in the `~naima.models` module.
+    The model accepts as a positional argument a `Naima <https://naima.readthedocs.io/en/latest/>`_
+    radiative model instance, used to compute the non-thermal emission from populations of
+    relativistic electrons or protons due to interactions with the ISM or with radiation and magnetic fields.
+
+    One of the advantages provided by this class consists in the possibility of performing a maximum
+    likelihood spectral fit of the model's parameters directly on observations, as opposed to the MCMC
+    `fit to flux points <https://naima.readthedocs.io/en/latest/mcmc.html>`_ featured in
+    Naima. All the parameters defining the parent population of charged particles are stored as
+    `~gammapy.utils.modeling.Parameter` and left free by default. In case that the radiative model is `
+    ~naima.radiative.Synchrotron`, the magnetic field strength may also be fitted. Parameters can be
+    freezed/unfreezed before the fit, and maximum/minimum values can be set to limit the parameters space to
+    the physically interesting region.
+
+    Parameters
+    ----------
+    radiative_model : `~naima.models.BaseRadiative`
+        An instance of a radiative model defined in `~naima.models`
+    distance : `~astropy.units.Quantity`, optional
+        Distance to the source. If set to 0, the intrinsic differential
+        luminosity will be returned. Default is 1 kpc
+    seed : str or list of str, optional
+        Seed photon field(s) to be considered for the `radiative_model` flux computation,
+        in case of a `~naima.models.InverseCompton` model. It can be a subset of the
+        `seed_photon_fields` list defining the `radiative_model`. Default is the whole list
+        of photon fields
+
+    Examples
+    --------
+    Create and plot a spectral model that convolves an `ExponentialCutoffPowerLaw` electron distribution
+    with an `InverseCompton` radiative model, in the presence of multiple seed photon fields.
+
+    .. plot::
+        :include-source:
+
+        import naima
+        from gammapy.spectrum.models import NaimaModel
+        import astropy.units as u
+        import matplotlib.pyplot as plt
+
+
+        particle_distribution = naima.models.ExponentialCutoffPowerLaw(1e30 / u.eV, 10 * u.TeV, 3.0, 30 * u.TeV)
+        radiative_model = naima.radiative.InverseCompton(
+            particle_distribution,
+            seed_photon_fields=[
+                "CMB",
+                ["FIR", 26.5 * u.K, 0.415 * u.eV / u.cm ** 3],
+            ],
+            Eemin=100 * u.GeV,
+        )
+
+        model = NaimaModel(radiative_model, distance=1.5 * u.kpc)
+
+        opts = {
+            "energy_range" : [10 * u.GeV, 80 * u.TeV],
+            "energy_power" : 2,
+            "flux_unit" : "erg-1 cm-2 s-1",
+        }
+
+        # Plot the total inverse Compton emission
+        model.plot(label='IC (total)', **opts)
+
+        # Plot the separate contributions from each seed photon field
+        for seed, ls in zip(['CMB','FIR'], ['-','--']):
+            model = NaimaModel(radiative_model, seed=seed, distance=1.5 * u.kpc)
+            model.plot(label="IC ({})".format(seed), ls=ls, color="gray", **opts)
+
+        plt.legend(loc='best')
+        plt.show()
+    """
+
+    # TODO: prevent users from setting new attributes after init
+    def __init__(self, radiative_model, distance=1.0 * u.kpc, seed=None):
+        import naima
+
+        self.radiative_model = radiative_model
+        self._particle_distribution = self.radiative_model.particle_distribution
+        self.distance = Parameter("distance", distance, frozen=True)
+        self.seed = seed
+
+        # This ensures the support of naima.models.TableModel
+        if isinstance(self._particle_distribution, naima.models.TableModel):
+            param_names = ["amplitude"]
+        else:
+            param_names = self._particle_distribution.param_names
+
+        parameters = []
+        for name in param_names:
+            value = getattr(self._particle_distribution, name)
+            setattr(self, name, Parameter(name, value))
+            parameters.append(getattr(self, name))
+
+        # In case of a synchrotron radiative model, append B to the fittable parameters
+        if "B" in self.radiative_model.param_names:
+            B = getattr(self.radiative_model, "B")
+            setattr(self, "B", Parameter("B", B))
+            parameters.append(getattr(self, "B"))
+
+        super().__init__(parameters)
+
+    def evaluate_error(self, energy):
+        # This method will need to be overridden here, since the radiative models in naima don't
+        # support the evaluation on energy values that is performed in the base class method
+        raise NotImplementedError(
+            "Error evaluation for naima models currently not supported."
+        )
+
+    def evaluate(self, energy, **kwargs):
+        """Evaluate the model"""
+        for name, value in kwargs.items():
+            setattr(self._particle_distribution, name, value)
+
+        distance = self.distance.quantity
+
+        # Flattening the input energy list and later reshaping the flux list
+        # prevents some radiative models from displaying broadcasting problems.
+        if self.seed is None:
+            dnde = self.radiative_model.flux(energy.flatten(), distance=distance)
+        else:
+            dnde = self.radiative_model.flux(
+                energy.flatten(), seed=self.seed, distance=distance
+            )
+
+        dnde = dnde.reshape(energy.shape)
+
+        unit = 1 / (energy.unit * u.cm ** 2 * u.s)
+        return dnde.to(unit)
