@@ -11,7 +11,6 @@
 * We might want to completely remove this module and expose the functionality
   directly on the spectral model classes some day.
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 
 __all__ = [
@@ -123,7 +122,6 @@ def power_law_energy_flux(I, g=g_DEFAULT, e=1, e1=1, e2=10):
     The analytical solution for the powerlaw case is given by:
 
     .. math::
-
         G(E_1, E_2) = I(\epsilon, \infty) \, \frac{1-\Gamma}
         {2-\Gamma} \, \frac{E_1^{2-\Gamma} - E_2^{2-\Gamma}}{\epsilon^{1-\Gamma}}
 
@@ -200,8 +198,7 @@ def power_law_f_from_points(e1, e2, f1, f2, e):
 def power_law_f_with_err(
     I_val=1, I_err=0, g_val=g_DEFAULT, g_err=0, e=1, e1=1, e2=E_INF
 ):
-    """Wrapper for f so the user doesn't have to know about
-    the uncertainties module"""
+    """Evaluate power-law ``dnde`` and propagate errors."""
     from uncertainties import unumpy
 
     I = unumpy.uarray(I_val, I_err)
@@ -215,8 +212,7 @@ def power_law_f_with_err(
 def power_law_I_with_err(
     f_val=1, f_err=0, g_val=g_DEFAULT, g_err=0, e=1, e1=1, e2=E_INF
 ):
-    """Wrapper for f so the user doesn't have to know about
-    the uncertainties module"""
+    """Evaluate power-law flux and propagate errors."""
     from uncertainties import unumpy
 
     f = unumpy.uarray(f_val, f_err)
