@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
@@ -8,7 +7,7 @@ from astropy.coordinates import SkyCoord, Angle
 from regions import CircleSkyRegion
 from ...utils.testing import assert_quantity_allclose
 from ...utils.testing import requires_dependency, requires_data
-from ...spectrum import SpectrumExtraction, SpectrumObservation
+from ...spectrum import SpectrumExtraction, SpectrumDatasetOnOff
 from ...background import ReflectedRegionsBackgroundEstimator
 from ...maps import WcsGeom, WcsNDMap
 from ...data import DataStore
@@ -65,8 +64,9 @@ def extraction(bkg_estimate, observations):
     )
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSpectrumExtraction:
+    @staticmethod
     @pytest.mark.parametrize(
         "pars, results",
         [
@@ -74,7 +74,7 @@ class TestSpectrumExtraction:
                 dict(containment_correction=False),
                 dict(
                     n_on=192,
-                    sigma=20.941125,
+                    sigma=20.866044,
                     aeff=580254.9 * u.m ** 2,
                     edisp=0.236176,
                     containment=1,
@@ -84,15 +84,15 @@ class TestSpectrumExtraction:
                 dict(containment_correction=True),
                 dict(
                     n_on=192,
-                    sigma=20.941125,
-                    aeff=373237.8 * u.m ** 2,
+                    sigma=20.866044,
+                    aeff=361924.746081 * u.m ** 2,
                     edisp=0.236176,
-                    containment=0.661611,
+                    containment=0.643835,
                 ),
             ),
         ],
     )
-    def test_extract(self, pars, results, observations, bkg_estimate):
+    def test_extract(pars, results, observations, bkg_estimate):
         """Test quantitative output for various configs"""
         extraction = SpectrumExtraction(
             observations=observations, bkg_estimate=bkg_estimate, **pars
@@ -130,18 +130,20 @@ class TestSpectrumExtraction:
     def test_run(tmpdir, extraction):
         """Test the run method and check if files are written correctly"""
         extraction.run()
-        extraction.write(outdir=tmpdir, overwrite=True)
-        testobs = SpectrumObservation.read(tmpdir / "ogip_data" / "pha_obs23523.fits")
+        extraction.write(outdir=str(tmpdir), overwrite=True)
+        testobs = SpectrumDatasetOnOff.from_ogip_files(
+            str(tmpdir / "ogip_data" / "pha_obs23523.fits")
+        )
         assert_quantity_allclose(
             testobs.aeff.data.data, extraction.spectrum_observations[0].aeff.data.data
         )
         assert_quantity_allclose(
-            testobs.on_vector.data.data,
-            extraction.spectrum_observations[0].on_vector.data.data,
+            testobs.counts.data.data,
+            extraction.spectrum_observations[0].counts.data.data,
         )
-        assert_quantity_allclose(
-            testobs.on_vector.energy.nodes,
-            extraction.spectrum_observations[0].on_vector.energy.nodes,
+        assert_allclose(
+            testobs.counts.energy.center,
+            extraction.spectrum_observations[0].counts.energy.center,
         )
 
     @requires_dependency("sherpa")
@@ -150,16 +152,16 @@ class TestSpectrumExtraction:
         import sherpa.astro.ui as sau
 
         extraction.run()
-        extraction.write(outdir=tmpdir, use_sherpa=True, overwrite=True)
+        extraction.write(outdir=str(tmpdir), use_sherpa=True, overwrite=True)
         sau.load_pha(str(tmpdir / "ogip_data" / "pha_obs23523.fits"))
         arf = sau.get_arf()
 
         actual = arf._arf._specresp
-        desired = extraction.spectrum_observations[0].aeff.data.data.value
+        desired = extraction.spectrum_observations[0].aeff.data.data.to_value("cm2")
         assert_allclose(actual, desired)
 
     def test_compute_energy_threshold(self, extraction):
         extraction.run()
         extraction.compute_energy_threshold(method_lo="area_max", area_percent_lo=10)
-        actual = extraction.spectrum_observations[0].lo_threshold
+        actual = extraction.spectrum_observations[0].energy_range[0]
         assert_quantity_allclose(actual, 0.8799225 * u.TeV, rtol=1e-3)
