@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 from numpy.testing import assert_allclose
 import numpy as np
@@ -33,7 +32,7 @@ def sens():
     return sens
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_cta_sensitivity_estimator(sens):
     table = sens.results_table
 
