@@ -1,12 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from astropy.units import Quantity
 
-__all__ = ["CountsPredictor", "integrate_spectrum"]
+__all__ = ["SpectrumEvaluator", "integrate_spectrum"]
 
 
-class CountsPredictor(object):
+class SpectrumEvaluator:
     """Calculate number of predicted counts (``npred``).
 
     The true and reconstructed energy binning are inferred from the provided IRFs.
@@ -26,19 +25,19 @@ class CountsPredictor(object):
 
     Examples
     --------
-    Calculate prediced counts in a desired reconstruced energy binning
+    Calculate predicted counts in a desired reconstruced energy binning
 
     .. plot::
         :include-source:
 
         from gammapy.irf import EnergyDispersion, EffectiveAreaTable
-        from gammapy.spectrum import models, CountsPredictor
+        from gammapy.spectrum import models, SpectrumEvaluator
         import numpy as np
         import astropy.units as u
         import matplotlib.pyplot as plt
 
-        e_true = np.logspace(-2,2.5,109) * u.TeV
-        e_reco = np.logspace(-2,2,73) * u.TeV
+        e_true = np.logspace(-2, 2.5, 109) * u.TeV
+        e_reco = np.logspace(-2, 2, 73) * u.TeV
 
         aeff = EffectiveAreaTable.from_parametrization(energy=e_true)
         edisp = EnergyDispersion.from_gauss(e_true=e_true, e_reco=e_reco,
@@ -50,12 +49,11 @@ class CountsPredictor(object):
 
         livetime = 1 * u.h
 
-        predictor = CountsPredictor(model=model,
-                                    aeff=aeff,
-                                    edisp=edisp,
-                                    livetime=livetime)
-        predictor.run()
-        predictor.npred.plot_hist()
+        predictor = SpectrumEvaluator(model=model,
+                                      aeff=aeff,
+                                      edisp=edisp,
+                                      livetime=livetime)
+        predictor.compute_npred().plot_hist()
         plt.show()
     """
 
@@ -67,14 +65,10 @@ class CountsPredictor(object):
         self.e_true = e_true
         self.e_reco = None
 
-        self.true_flux = None
-        self.true_counts = None
-        self.npred = None
-
-    def run(self):
-        self.integrate_model()
-        self.apply_aeff()
-        self.apply_edisp()
+    def compute_npred(self):
+        integral_flux = self.integrate_model()
+        true_counts = self.apply_aeff(integral_flux)
+        return self.apply_edisp(true_counts)
 
     def integrate_model(self):
         """Integrate model in true energy space"""
@@ -87,38 +81,38 @@ class CountsPredictor(object):
                         ref_unit = unit
             except IndexError:
                 ref_unit = "TeV"
-            self.e_true = self.aeff.energy.bins.to(ref_unit)
+            self.e_true = self.aeff.energy.edges.to(ref_unit)
         else:
             if self.e_true is None:
                 raise ValueError("No true energy binning given")
 
-        self.true_flux = self.model.integral(
+        return self.model.integral(
             emin=self.e_true[:-1], emax=self.e_true[1:], intervals=True
         )
 
-    def apply_aeff(self):
+    def apply_aeff(self, integral_flux):
         if self.aeff is not None:
-            cts = self.true_flux * self.aeff.data.data
+            cts = integral_flux * self.aeff.data.data
         else:
-            cts = self.true_flux
+            cts = integral_flux
 
         # Multiply with livetime if not already contained in aeff or model
         if cts.unit.is_equivalent("s-1"):
             cts *= self.livetime
 
-        self.true_counts = cts.to("")
+        return cts.to("")
 
-    def apply_edisp(self):
+    def apply_edisp(self, true_counts):
         from . import CountsSpectrum
 
         if self.edisp is not None:
-            cts = self.edisp.apply(self.true_counts)
-            self.e_reco = self.edisp.e_reco.bins
+            cts = self.edisp.apply(true_counts)
+            self.e_reco = self.edisp.e_reco.edges
         else:
-            cts = self.true_counts
+            cts = true_counts
             self.e_reco = self.e_true
 
-        self.npred = CountsSpectrum(
+        return CountsSpectrum(
             data=cts, energy_lo=self.e_reco[:-1], energy_hi=self.e_reco[1:]
         )
 
@@ -156,7 +150,7 @@ def integrate_spectrum(func, xmin, xmax, ndecade=100, intervals=False):
     if np.isscalar(xmin):
         logmin = np.log10(xmin)
         logmax = np.log10(xmax)
-        n = (logmax - logmin) * ndecade
+        n = int((logmax - logmin) * ndecade)
         x = np.logspace(logmin, logmax, n)
     else:
         x = np.append(xmin, xmax[-1])
