@@ -1,11 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 from collections import OrderedDict
 import numpy as np
 import astropy.units as u
 from astropy.table import Table
 from astropy.time import Time
-from ..spectrum.utils import CountsPredictor
+from ..spectrum.utils import SpectrumEvaluator
 from ..stats.poisson import excess_error, excess_ul_helene
 from ..utils.scripts import make_path
 from ..utils.table import table_from_row_data
@@ -14,7 +13,7 @@ from ..stats.poisson import significance_on_off
 __all__ = ["LightCurve", "LightCurveEstimator"]
 
 
-class LightCurve(object):
+class LightCurve:
     """Lightcurve container.
 
     The lightcurve data is stored in ``table``.
@@ -226,7 +225,7 @@ class LightCurve(object):
 
         ax.errorbar(x=x, y=y, xerr=xerr, yerr=yerr, uplims=is_ul, **kwargs)
         ax.set_xlabel("Time ({})".format(time_format.upper()))
-        ax.set_ylabel("Flux ({0:FITS})".format(u.Unit(flux_unit)))
+        ax.set_ylabel("Flux ({:FITS})".format(u.Unit(flux_unit)))
         if time_format == "iso":
             ax.xaxis.set_major_formatter(DateFormatter("%Y-%m-%d %H:%M:%S"))
             plt.setp(
@@ -341,14 +340,12 @@ class LightCurve(object):
             x = x.mjd
             xn, xp = xn.to("d").value, xp.to("d").value
         else:
-            raise ValueError(
-                "The time format '{}' is not supported.".format(time_format)
-            )
+            raise ValueError("Invalid time_format: {}".format(time_format))
 
         return x, (xn, xp)
 
 
-class LightCurveEstimator(object):
+class LightCurveEstimator:
     """Light curve estimator.
 
     For a usage example see :gp-notebook:`light_curve`.
@@ -461,7 +458,7 @@ class LightCurveEstimator(object):
         off = self.off_evt_list[t_index]
         on = self.on_evt_list[t_index]
         # introduce the e_reco binning here
-        e_reco = spec.e_reco
+        e_reco = spec.counts.energy.edges
         if energy_range is not None:
             emin = e_reco[e_reco.searchsorted(max(spec.lo_threshold, energy_range[0]))]
             emax = e_reco[
@@ -484,7 +481,7 @@ class LightCurveEstimator(object):
 
     @staticmethod
     def _alpha(time_holder, obs_properties, n, istart, i):
-        """ Helper function for make_time_intervals_min_significance
+        """Helper function for make_time_intervals_min_significance.
 
         Parameters
         ----------
@@ -816,14 +813,15 @@ class LightCurveEstimator(object):
                     )  # user
                 )
             )[0]
-            counts_predictor = CountsPredictor(
+            counts_predictor = SpectrumEvaluator(
                 livetime=livetime_to_add,
                 aeff=spec.aeff,
                 edisp=spec.edisp,
                 model=spectral_model,
             )
-            counts_predictor.run()
-            counts_predicted_excess = counts_predictor.npred.data.data[e_idx[:-1]]
+            counts_predicted_excess = counts_predictor.compute_npred().data.data[
+                e_idx[:-1]
+            ]
 
             obs_predicted_excess = np.sum(counts_predicted_excess)
 
