@@ -1,12 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from ...utils.random import get_random_state
 
 __all__ = ["GeneralRandom"]
 
 
-class GeneralRandom(object):
+class GeneralRandom:
     """Fast random number generation with an arbitrary pdf
     of a continuous variable x.
     Linear interpolation is applied between points pdf(x)
@@ -21,9 +20,10 @@ class GeneralRandom(object):
 
     Note: Should it be required the cdf could be deleted
     after computing to inversecdf to free memory since it is
-    not required for random number generation."""
+    not required for random number generation.
+    """
 
-    def __init__(self, pdf, min_range, max_range, ninversecdf=None, ran_res=1e3):
+    def __init__(self, pdf, min_range, max_range, ninversecdf=None, ran_res=1000):
         """Initialize the lookup table
 
         Inputs:
@@ -109,7 +109,8 @@ class GeneralRandom(object):
         """Plot the pdf, cdf and inversecdf
         and a random distribution of sample size N.
 
-        Useful for illustrating the interpolation and debugging."""
+        Useful for illustrating the interpolation and debugging.
+        """
         import matplotlib.pyplot as plt
 
         # Plot the cdf
