@@ -1,13 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Implementation of the GeneralRandomArray class"""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import numpy as np
 from ...utils.random import get_random_state
 
 __all__ = ["GeneralRandomArray"]
 
 
-class GeneralRandomArray(object):
+class GeneralRandomArray:
     """Draw random indices from a discrete probability distribution
     given by a numpy array.
     The array dimension can be arbitrary.
