@@ -1,10 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 import numpy as np
 from astropy.units import Quantity
 from astropy.io import fits
-from ..extern import six
 
 __all__ = ["Energy", "EnergyBounds"]
 
@@ -39,13 +37,13 @@ class Energy(Quantity):
         # Techniques to subclass Quantity taken from astropy.coordinates.Angle
         # see: http://docs.scipy.org/doc/numpy/user/basics.subclassing.html
 
-        if isinstance(energy, six.string_types):
+        if isinstance(energy, str):
             val, unit = energy.split()
             energy = float(val)
 
         # This is a pylint error false positive
         # See https://github.com/PyCQA/pylint/issues/2335#issuecomment-415055075
-        self = super(Energy, cls).__new__(
+        self = super().__new__(
             cls,
             energy,
             unit,  # pylint:disable=redundant-keyword-arg
@@ -61,13 +59,13 @@ class Energy(Quantity):
         return self
 
     def __array_finalize__(self, obj):
-        super(Energy, self).__array_finalize__(obj)
+        super().__array_finalize__(obj)
 
     def __quantity_subclass__(self, unit):
         if unit.is_equivalent("eV"):
             return Energy, True
         else:
-            return super(Energy, self).__quantity_subclass__(unit)[0], False
+            return super().__quantity_subclass__(unit)[0], False
 
     @property
     def nbins(self):
@@ -253,7 +251,7 @@ class EnergyBounds(Energy):
         unit : `~astropy.units.UnitBase`, str, None
             Energy unit
         """
-        return super(EnergyBounds, cls).equal_log_spacing(emin, emax, nbins + 1, unit)
+        return super().equal_log_spacing(emin, emax, nbins + 1, unit)
 
     @classmethod
     def from_ebounds(cls, hdu):
