@@ -191,7 +191,6 @@ For further information on Astropy, see the Astropy docs at
 We will have to see if / what we need here in `gammapy.utils.fits`
 as a stable and nice interface on top of what Astropy provides.
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 from collections import OrderedDict
 import numpy as np
 from astropy.io import fits
@@ -206,7 +205,7 @@ __all__ = ["SmartHDUList", "energy_axis_to_ebounds", "earth_location_from_dict"]
 
 # TODO: decide what to call this class.
 # Would `FITSFile` be better than `SmartHDUList`?
-class SmartHDUList(object):
+class SmartHDUList:
     """A FITS HDU list wrapper with some sugar.
 
     This is a thin wrapper around `~astropy.io.fits.HDUList`,
@@ -219,14 +218,13 @@ class SmartHDUList(object):
 
     Examples
     --------
-
     Opening a SmartHDUList calls `astropy.io.fits.open` to get a `~astropy.io.fits.HDUList`
     object, and then stores it away in the ``hdu_list`` attribute:
 
     >>> from gammapy.utils.fits import SmartHDUList
     >>> hdus = SmartHDUList.open('$GAMMAPY_DATA/catalogs/fermi/gll_psch_v08.fit.gz')
     >>> type(hdus.hdu_list)
-    astropy.io.fits.hdu.hdulist.HDUList
+    <class 'astropy.io.fits.hdu.hdulist.HDUList'>
 
     So of course, you can do the usual things via ``hdus.hdu_list``:
 
