@@ -1,6 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """iminuit fitting functions."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 import numpy as np
 from .likelihood import Likelihood
@@ -15,7 +14,7 @@ class MinuitLikelihood(Likelihood):
 
     def fcn(self, *factors):
         self.parameters.set_parameter_factors(factors)
-        return self.function(self.parameters)
+        return self.function()
 
 
 def optimize_iminuit(parameters, function, **kwargs):
@@ -28,7 +27,8 @@ def optimize_iminuit(parameters, function, **kwargs):
     function : callable
         Likelihood function
     **kwargs : dict
-        Options passed to `iminuit.Minuit` constructor
+        Options passed to `iminuit.Minuit` constructor. If there is an entry 'migrad_opts', those options
+        will be passed to `iminuit.Minuit.migrad()`.
 
     Returns
     -------
@@ -45,8 +45,10 @@ def optimize_iminuit(parameters, function, **kwargs):
 
     minuit_func = MinuitLikelihood(function, parameters)
 
+    kwargs = kwargs.copy()
+    migrad_opts = kwargs.pop("migrad_opts", {})
     minuit = Minuit(minuit_func.fcn, **kwargs)
-    minuit.migrad()
+    minuit.migrad(**migrad_opts)
 
     factors = minuit.args
     info = {
@@ -55,6 +57,7 @@ def optimize_iminuit(parameters, function, **kwargs):
         "message": _get_message(minuit),
     }
     optimizer = minuit
+
     return factors, info, optimizer
 
 
@@ -63,8 +66,8 @@ def covariance_iminuit(minuit):
 
     message, success = "Hesse terminated successfully.", True
     try:
-        covariance_factors = _get_covariance(minuit)
-    except TypeError:
+        covariance_factors = minuit.np_covariance()
+    except (TypeError, RuntimeError):
         N = len(minuit.args)
         covariance_factors = np.nan * np.ones((N, N))
         message, success = "Hesse failed", False
@@ -74,8 +77,9 @@ def covariance_iminuit(minuit):
 def confidence_iminuit(minuit, parameters, parameter, sigma, maxcall=0):
     # TODO: this is ugly - design something better for translating to MINUIT parameter names.
     # Maybe a wrapper class MinuitParameters?
-    idx = parameters._get_idx(parameter)
-    var = _make_parname(idx, parameters[idx])
+    parameter = parameters[parameter]
+    idx = parameters.free_parameters.index(parameter)
+    var = _make_parname(idx, parameter)
 
     message, success = "Minos terminated successfully.", True
     try:
@@ -142,32 +146,28 @@ def make_minuit_par_kwargs(parameters):
 
     See: http://iminuit.readthedocs.io/en/latest/api.html#iminuit.Minuit
     """
-    names = _make_parnames(parameters)
+    names = _make_parnames(parameters.free_parameters)
     kwargs = {"forced_parameters": names}
 
-    for name, par in zip(names, parameters):
+    for name, par in zip(names, parameters.free_parameters):
         kwargs[name] = par.factor
 
         min_ = None if np.isnan(par.factor_min) else par.factor_min
         max_ = None if np.isnan(par.factor_max) else par.factor_max
         kwargs["limit_{}".format(name)] = (min_, max_)
 
-        kwargs["error_{}".format(name)] = 1
-        kwargs["fix_{}".format(name)] = par.frozen
+        if parameters.covariance is not None:
+            error = parameters.error(par) / par.scale
+        elif parameters.apply_autoscale:
+            error = 1
+        else:
+            error = 1
+            log.warning(
+                "Neither covariance matrix set nor auto-scaling of parameters activated."
+                "Assuming stepsize of 1, which could lead to convergence problems of the "
+                "Minuit optimizer."
+            )
+
+        kwargs["error_{}".format(name)] = error
 
     return kwargs
-
-
-def _get_covariance(minuit):
-    """Get full covariance matrix as Numpy array.
-
-    This was added as `minuit.np_covariance` in `iminuit` in v1.3,
-    but we still want to support v1.2
-    """
-    n = len(minuit.parameters)
-    m = np.zeros((n, n))
-    for i1, k1 in enumerate(minuit.parameters):
-        for i2, k2 in enumerate(minuit.parameters):
-            if {k1, k2} <= set(minuit.list_of_vary_param()):
-                m[i1, i2] = minuit.covariance[(k1, k2)]
-    return m
