@@ -1,19 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Model parameter classes."""
-from __future__ import absolute_import, division, print_function, unicode_literals
 import copy
 import numpy as np
-from ...extern import six
 from astropy import units as u
+from astropy.units.core import UnitConversionError
 from astropy.table import Table
 from ..array import check_type
 
 __all__ = ["Parameter", "Parameters"]
 
 
-class Parameter(object):
-    """
-    Class representing model parameters.
+class Parameter:
+    """A model parameter.
 
     Note that the parameter value has been split into
     a factor and scale like this::
@@ -55,8 +53,10 @@ class Parameter(object):
         self.name = name
         self.scale = scale
 
-        if isinstance(factor, u.Quantity) or isinstance(factor, six.string_types):
-            self.quantity = factor
+        if isinstance(factor, u.Quantity) or isinstance(factor, str):
+            val = u.Quantity(factor)
+            self.value = val.value
+            self.unit = val.unit
         else:
             self.factor = factor
             self.unit = unit
@@ -161,8 +161,16 @@ class Parameter(object):
     @quantity.setter
     def quantity(self, val):
         val = u.Quantity(val)
-        self.value = val.value
-        self.unit = val.unit
+        try:
+            val.to(self.unit)
+            self.value = val.value
+            self.unit = val.unit
+        except UnitConversionError:
+            raise UnitConversionError(
+                "{0} parameter must have units homogeneous with {1}".format(
+                    self.name, self.unit
+                )
+            )
 
     def __repr__(self):
         return (
@@ -215,10 +223,10 @@ class Parameter(object):
             raise ValueError("Invalid method: {}".format(method))
 
 
-class Parameters(object):
+class Parameters:
     """List of `Parameter`.
 
-    Holds covariance matrix
+    Holds covariance matrix.
 
     Parameters
     ----------
@@ -232,11 +240,25 @@ class Parameters(object):
         see `~gammapy.utils.modeling.Parameter.autoscale`
     """
 
-    def __init__(self, parameters, covariance=None, apply_autoscale=True):
-        self._parameters = parameters
+    def __init__(self, parameters=None, covariance=None, apply_autoscale=True):
+        if parameters is None:
+            parameters = []
+
+        self._parameters = self._filter_unique_parameters(parameters)
         self.covariance = covariance
         self.apply_autoscale = apply_autoscale
 
+    @staticmethod
+    def _filter_unique_parameters(parameters):
+        """Filter unique parameters from a list of parameters"""
+        unique_parameters = []
+
+        for par in parameters:
+            if par not in unique_parameters:
+                unique_parameters.append(par)
+
+        return unique_parameters
+
     def _init_covariance(self):
         if self.covariance is None:
             shape = (len(self.parameters), len(self.parameters))
@@ -251,6 +273,11 @@ class Parameters(object):
         """List of `Parameter`."""
         return self._parameters
 
+    @property
+    def free_parameters(self):
+        """List of free parameters"""
+        return [par for par in self.parameters if not par.frozen]
+
     # TODO: replace this with a better API to update parameters
     @parameters.setter
     def parameters(self, vals):
@@ -274,14 +301,11 @@ class Parameters(object):
         The input can be a parameter object, parameter name (str)
         or if a parameter index (int) is passed in, it is simply returned.
         """
-        if isinstance(val, six.integer_types):
+        if isinstance(val, int):
             return val
         elif isinstance(val, Parameter):
-            for idx, par in enumerate(self.parameters):
-                if val is par:
-                    return idx
-            raise IndexError("No parameter: {!r}".format(val))
-        elif isinstance(val, six.string_types):
+            return self.parameters.index(val)
+        elif isinstance(val, str):
             for idx, par in enumerate(self.parameters):
                 if val == par.name:
                     return idx
@@ -358,9 +382,7 @@ class Parameters(object):
 
     @property
     def _ufloats(self):
-        """
-        Return dict of ufloats with covariance
-        """
+        """Return dict of ufloats with covariance."""
         from uncertainties import correlated_values
 
         values = [_.value for _ in self.parameters]
@@ -379,8 +401,7 @@ class Parameters(object):
 
     # TODO: deprecate or remove this?
     def set_parameter_errors(self, errors):
-        """
-        Set uncorrelated parameters errors.
+        """Set uncorrelated parameters errors.
 
         Parameters
         ----------
@@ -445,19 +466,34 @@ class Parameters(object):
 
         Used in the optimizer interface.
         """
-        for factor, parameter in zip(factors, self.parameters):
-            parameter.factor = factor
+        idx = 0
+        for parameter in self.parameters:
+            if not parameter.frozen:
+                parameter.factor = factors[idx]
+                idx += 1
 
     @property
     def _scale_matrix(self):
         scales = [par.scale for par in self.parameters]
         return np.outer(scales, scales)
 
+    def _expand_factor_matrix(self, matrix):
+        """Expand covariance matrix with zeros for frozen parameters"""
+        shape = (len(self.parameters), len(self.parameters))
+        matrix_expanded = np.zeros(shape)
+        mask = np.array([par.frozen for par in self.parameters])
+        free_parameters = ~(mask | mask[:, np.newaxis])
+        matrix_expanded[free_parameters] = matrix.ravel()
+        return matrix_expanded
+
     def set_covariance_factors(self, matrix):
         """Set covariance from factor covariance matrix.
 
         Used in the optimizer interface.
         """
+        if not np.sqrt(matrix.size) == len(self.parameters):
+            matrix = self._expand_factor_matrix(matrix)
+
         self.covariance = self._scale_matrix * matrix
 
     def autoscale(self, method="scale10"):
@@ -485,24 +521,29 @@ class Parameters(object):
         ::
 
             from gammapy.spectrum.models import PowerLaw
-
             pwl = PowerLaw(index=2)
             with pwl.parameters.restore_values:
                 pwl.parameters["index"].value = 3
-
             print(pwl.parameters["index"].value)
         """
         return restore_parameters_values(self)
 
+    def freeze_all(self):
+        """Freeze all parameters"""
+        for par in self.parameters:
+            par.frozen = True
+
 
 class restore_parameters_values:
     def __init__(self, parameters):
         self.parameters = parameters
         self.values = [_.value for _ in parameters]
+        self.frozen = [_.frozen for _ in parameters]
 
     def __enter__(self):
         pass
 
     def __exit__(self, type, value, traceback):
-        for value, par in zip(self.values, self.parameters):
+        for value, par, frozen in zip(self.values, self.parameters, self.frozen):
             par.value = value
+            par.frozen = frozen
